#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database


_sym_db = _symbol_database.Default()


from google.appengine.datastore import datastore_v3_bytes_pb2 as google_dot_appengine_dot_datastore_dot_datastore__v3__bytes__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<google/appengine/api/taskqueue/taskqueue_service_bytes.proto\x12\x10\x61pphosting_bytes\x1a\x33google/appengine/datastore/datastore_v3_bytes.proto\"\xa0\n\n\x15TaskQueueServiceError\"\x86\n\n\tErrorCode\x12\x06\n\x02OK\x10\x00\x12\x11\n\rUNKNOWN_QUEUE\x10\x01\x12\x13\n\x0fTRANSIENT_ERROR\x10\x02\x12\x12\n\x0eINTERNAL_ERROR\x10\x03\x12\x12\n\x0eTASK_TOO_LARGE\x10\x04\x12\x15\n\x11INVALID_TASK_NAME\x10\x05\x12\x16\n\x12INVALID_QUEUE_NAME\x10\x06\x12\x0f\n\x0bINVALID_URL\x10\x07\x12\x16\n\x12INVALID_QUEUE_RATE\x10\x08\x12\x15\n\x11PERMISSION_DENIED\x10\t\x12\x17\n\x13TASK_ALREADY_EXISTS\x10\n\x12\x13\n\x0fTOMBSTONED_TASK\x10\x0b\x12\x0f\n\x0bINVALID_ETA\x10\x0c\x12\x13\n\x0fINVALID_REQUEST\x10\r\x12\x10\n\x0cUNKNOWN_TASK\x10\x0e\x12\x14\n\x10TOMBSTONED_QUEUE\x10\x0f\x12\x17\n\x13\x44UPLICATE_TASK_NAME\x10\x10\x12\x0b\n\x07SKIPPED\x10\x11\x12\x12\n\x0eTOO_MANY_TASKS\x10\x12\x12\x13\n\x0fINVALID_PAYLOAD\x10\x13\x12\x1c\n\x18INVALID_RETRY_PARAMETERS\x10\x14\x12\x16\n\x12INVALID_QUEUE_MODE\x10\x15\x12\x14\n\x10\x41\x43L_LOOKUP_ERROR\x10\x16\x12#\n\x1fTRANSACTIONAL_REQUEST_TOO_LARGE\x10\x17\x12\x1a\n\x16INCORRECT_CREATOR_NAME\x10\x18\x12\x16\n\x12TASK_LEASE_EXPIRED\x10\x19\x12\x10\n\x0cQUEUE_PAUSED\x10\x1a\x12\x0f\n\x0bINVALID_TAG\x10\x1b\x12\x1a\n\x16INVALID_LOGGING_CONFIG\x10\x1c\x12\x1d\n\x19INVALID_DISPATCH_DEADLINE\x10\x1d\x12\x14\n\x0f\x44\x41TASTORE_ERROR\x10\x90N\x12\x1a\n\x15\x44\x41TASTORE_BAD_REQUEST\x10\x91N\x12%\n DATASTORE_CONCURRENT_TRANSACTION\x10\x92N\x12\x1d\n\x18\x44\x41TASTORE_INTERNAL_ERROR\x10\x93N\x12\x19\n\x14\x44\x41TASTORE_NEED_INDEX\x10\x94N\x12\x16\n\x11\x44\x41TASTORE_TIMEOUT\x10\x95N\x12 \n\x1b\x44\x41TASTORE_PERMISSION_DENIED\x10\x96N\x12\x1d\n\x18\x44\x41TASTORE_BIGTABLE_ERROR\x10\x97N\x12+\n&DATASTORE_COMMITTED_BUT_STILL_APPLYING\x10\x98N\x12\"\n\x1d\x44\x41TASTORE_CAPABILITY_DISABLED\x10\x99N\x12$\n\x1f\x44\x41TASTORE_TRY_ALTERNATE_BACKEND\x10\x9aN\x12 \n\x1b\x44\x41TASTORE_SAFE_TIME_TOO_OLD\x10\x9bN\x12!\n\x1c\x44\x41TASTORE_RESOURCE_EXHAUSTED\x10\x9cN\x12\x18\n\x13\x44\x41TASTORE_NOT_FOUND\x10\x9dN\x12\x1d\n\x18\x44\x41TASTORE_ALREADY_EXISTS\x10\x9eN\x12\"\n\x1d\x44\x41TASTORE_FAILED_PRECONDITION\x10\x9fN\x12\x1e\n\x19\x44\x41TASTORE_UNAUTHENTICATED\x10\xa0N\x12\x16\n\x11\x44\x41TASTORE_ABORTED\x10\xa1N\x12\'\n\"DATASTORE_SNAPSHOT_VERSION_TOO_OLD\x10\xa2N\"\x1b\n\x0bTaskPayload*\x08\x08\n\x10\xff\xff\xff\xff\x07:\x02\x08\x01\"\x9e\x01\n\x18TaskQueueRetryParameters\x12\x13\n\x0bretry_limit\x18\x01 \x01(\x05\x12\x15\n\rage_limit_sec\x18\x02 \x01(\x03\x12\x1c\n\x0fmin_backoff_sec\x18\x03 \x01(\x01:\x03\x30.1\x12\x1d\n\x0fmax_backoff_sec\x18\x04 \x01(\x01:\x04\x33\x36\x30\x30\x12\x19\n\rmax_doublings\x18\x05 \x01(\x05:\x02\x31\x36\"8\n\x0cTaskQueueAcl\x12\x12\n\nuser_email\x18\x01 \x03(\x0c\x12\x14\n\x0cwriter_email\x18\x02 \x03(\x0c\"1\n\x13TaskQueueHttpHeader\x12\x0b\n\x03key\x18\x01 \x02(\x0c\x12\r\n\x05value\x18\x02 \x02(\x0c\"+\n\rTaskQueueMode\"\x1a\n\x04Mode\x12\x08\n\x04PUSH\x10\x00\x12\x08\n\x04PULL\x10\x01\"\x87\x07\n\x13TaskQueueAddRequest\x12\x12\n\nqueue_name\x18\x01 \x02(\x0c\x12\x11\n\ttask_name\x18\x02 \x02(\x0c\x12\x10\n\x08\x65ta_usec\x18\x03 \x02(\x03\x12I\n\x06method\x18\x05 \x01(\x0e\x32\x33.apphosting_bytes.TaskQueueAddRequest.RequestMethod:\x04POST\x12\x0b\n\x03url\x18\x04 \x01(\x0c\x12<\n\x06header\x18\x06 \x03(\n2,.apphosting_bytes.TaskQueueAddRequest.Header\x12\x10\n\x04\x62ody\x18\t \x01(\x0c\x42\x02\x08\x01\x12?\n\x0btransaction\x18\n \x01(\x0b\x32*.apphosting_datastore_v3_bytes.Transaction\x12\x1d\n\x15\x64\x61tastore_transaction\x18\x15 \x01(\x0c\x12\x0e\n\x06\x61pp_id\x18\x0b \x01(\x0c\x12J\n\rcrontimetable\x18\x0c \x01(\n23.apphosting_bytes.TaskQueueAddRequest.CronTimetable\x12\x13\n\x0b\x64\x65scription\x18\x0f \x01(\x0c\x12.\n\x07payload\x18\x10 \x01(\x0b\x32\x1d.apphosting_bytes.TaskPayload\x12\x44\n\x10retry_parameters\x18\x11 \x01(\x0b\x32*.apphosting_bytes.TaskQueueRetryParameters\x12\x32\n\x04mode\x18\x12 \x01(\x0e\x32$.apphosting_bytes.TaskQueueMode.Mode\x12\x0b\n\x03tag\x18\x13 \x01(\x0c\x12I\n\x15\x63ron_retry_parameters\x18\x14 \x01(\x0b\x32*.apphosting_bytes.TaskQueueRetryParameters\x12\x1e\n\x16\x64ispatch_deadline_usec\x18\x16 \x01(\x03\x1a$\n\x06Header\x12\x0b\n\x03key\x18\x07 \x02(\x0c\x12\r\n\x05value\x18\x08 \x02(\x0c\x1a\x33\n\rCronTimetable\x12\x10\n\x08schedule\x18\r \x02(\x0c\x12\x10\n\x08timezone\x18\x0e \x02(\x0c\"A\n\rRequestMethod\x12\x07\n\x03GET\x10\x01\x12\x08\n\x04POST\x10\x02\x12\x08\n\x04HEAD\x10\x03\x12\x07\n\x03PUT\x10\x04\x12\n\n\x06\x44\x45LETE\x10\x05\"0\n\x14TaskQueueAddResponse\x12\x18\n\x10\x63hosen_task_name\x18\x01 \x01(\x0c\"U\n\x17TaskQueueBulkAddRequest\x12:\n\x0b\x61\x64\x64_request\x18\x01 \x03(\x0b\x32%.apphosting_bytes.TaskQueueAddRequest\"\xd0\x01\n\x18TaskQueueBulkAddResponse\x12I\n\ntaskresult\x18\x01 \x03(\n25.apphosting_bytes.TaskQueueBulkAddResponse.TaskResult\x1ai\n\nTaskResult\x12\x41\n\x06result\x18\x02 \x02(\x0e\x32\x31.apphosting_bytes.TaskQueueServiceError.ErrorCode\x12\x18\n\x10\x63hosen_task_name\x18\x03 \x01(\x0c\"O\n\x16TaskQueueDeleteRequest\x12\x12\n\nqueue_name\x18\x01 \x02(\x0c\x12\x11\n\ttask_name\x18\x02 \x03(\x0c\x12\x0e\n\x06\x61pp_id\x18\x03 \x01(\x0c\"\\\n\x17TaskQueueDeleteResponse\x12\x41\n\x06result\x18\x03 \x03(\x0e\x32\x31.apphosting_bytes.TaskQueueServiceError.ErrorCode\"Q\n\x18TaskQueueForceRunRequest\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\x0c\x12\x12\n\nqueue_name\x18\x02 \x02(\x0c\x12\x11\n\ttask_name\x18\x03 \x02(\x0c\"^\n\x19TaskQueueForceRunResponse\x12\x41\n\x06result\x18\x03 \x02(\x0e\x32\x31.apphosting_bytes.TaskQueueServiceError.ErrorCode\"\xa1\x03\n\x1bTaskQueueUpdateQueueRequest\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\x0c\x12\x12\n\nqueue_name\x18\x02 \x02(\x0c\x12 \n\x18\x62ucket_refill_per_second\x18\x03 \x02(\x01\x12\x17\n\x0f\x62ucket_capacity\x18\x04 \x02(\x05\x12\x1b\n\x13user_specified_rate\x18\x05 \x01(\t\x12\x44\n\x10retry_parameters\x18\x06 \x01(\x0b\x32*.apphosting_bytes.TaskQueueRetryParameters\x12\x1f\n\x17max_concurrent_requests\x18\x07 \x01(\x05\x12\x32\n\x04mode\x18\x08 \x01(\x0e\x32$.apphosting_bytes.TaskQueueMode.Mode\x12+\n\x03\x61\x63l\x18\t \x01(\x0b\x32\x1e.apphosting_bytes.TaskQueueAcl\x12>\n\x0fheader_override\x18\n \x03(\x0b\x32%.apphosting_bytes.TaskQueueHttpHeader\"\x1e\n\x1cTaskQueueUpdateQueueResponse\"?\n\x1bTaskQueueFetchQueuesRequest\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\x0c\x12\x10\n\x08max_rows\x18\x02 \x02(\x05\"\x97\x04\n\x1cTaskQueueFetchQueuesResponse\x12\x43\n\x05queue\x18\x01 \x03(\n24.apphosting_bytes.TaskQueueFetchQueuesResponse.Queue\x1a\xb1\x03\n\x05Queue\x12\x12\n\nqueue_name\x18\x02 \x02(\x0c\x12 \n\x18\x62ucket_refill_per_second\x18\x03 \x02(\x01\x12\x17\n\x0f\x62ucket_capacity\x18\x04 \x02(\x01\x12\x1b\n\x13user_specified_rate\x18\x05 \x01(\t\x12\x0e\n\x06paused\x18\x06 \x02(\x08\x12\x44\n\x10retry_parameters\x18\x07 \x01(\x0b\x32*.apphosting_bytes.TaskQueueRetryParameters\x12\x1f\n\x17max_concurrent_requests\x18\x08 \x01(\x05\x12\x32\n\x04mode\x18\t \x01(\x0e\x32$.apphosting_bytes.TaskQueueMode.Mode\x12+\n\x03\x61\x63l\x18\n \x01(\x0b\x32\x1e.apphosting_bytes.TaskQueueAcl\x12>\n\x0fheader_override\x18\x0b \x03(\x0b\x32%.apphosting_bytes.TaskQueueHttpHeader\x12$\n\x0c\x63reator_name\x18\x0c \x01(\t:\napphostingB\x02\x08\x01\"\\\n\x1fTaskQueueFetchQueueStatsRequest\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\x0c\x12\x12\n\nqueue_name\x18\x02 \x03(\x0c\x12\x15\n\rmax_num_tasks\x18\x03 \x01(\x05\"\xab\x01\n\x19TaskQueueScannerQueueInfo\x12\x1c\n\x14\x65xecuted_last_minute\x18\x01 \x02(\x03\x12\x1a\n\x12\x65xecuted_last_hour\x18\x02 \x02(\x03\x12!\n\x19sampling_duration_seconds\x18\x03 \x02(\x01\x12\x1a\n\x12requests_in_flight\x18\x04 \x01(\x05\x12\x15\n\renforced_rate\x18\x05 \x01(\x01\"\xf2\x01\n TaskQueueFetchQueueStatsResponse\x12Q\n\nqueuestats\x18\x01 \x03(\n2=.apphosting_bytes.TaskQueueFetchQueueStatsResponse.QueueStats\x1a{\n\nQueueStats\x12\x11\n\tnum_tasks\x18\x02 \x02(\x05\x12\x17\n\x0foldest_eta_usec\x18\x03 \x02(\x03\x12\x41\n\x0cscanner_info\x18\x04 \x01(\x0b\x32+.apphosting_bytes.TaskQueueScannerQueueInfo\"O\n\x1aTaskQueuePauseQueueRequest\x12\x0e\n\x06\x61pp_id\x18\x01 \x02(\x0c\x12\x12\n\nqueue_name\x18\x02 \x02(\x0c\x12\r\n\x05pause\x18\x03 \x02(\x08\"\x1d\n\x1bTaskQueuePauseQueueResponse\"@\n\x1aTaskQueuePurgeQueueRequest\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\x0c\x12\x12\n\nqueue_name\x18\x02 \x02(\x0c\"\x1d\n\x1bTaskQueuePurgeQueueResponse\"A\n\x1bTaskQueueDeleteQueueRequest\x12\x0e\n\x06\x61pp_id\x18\x01 \x02(\x0c\x12\x12\n\nqueue_name\x18\x02 \x02(\x0c\"\x1e\n\x1cTaskQueueDeleteQueueResponse\"-\n\x1bTaskQueueDeleteGroupRequest\x12\x0e\n\x06\x61pp_id\x18\x01 \x02(\x0c\"\x1e\n\x1cTaskQueueDeleteGroupResponse\"\x99\x01\n\x1aTaskQueueQueryTasksRequest\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\x0c\x12\x12\n\nqueue_name\x18\x02 \x02(\x0c\x12\x17\n\x0fstart_task_name\x18\x03 \x01(\x0c\x12\x16\n\x0estart_eta_usec\x18\x04 \x01(\x03\x12\x11\n\tstart_tag\x18\x06 \x01(\x0c\x12\x13\n\x08max_rows\x18\x05 \x01(\x05:\x01\x31\"\xb0\x08\n\x1bTaskQueueQueryTasksResponse\x12@\n\x04task\x18\x01 \x03(\n22.apphosting_bytes.TaskQueueQueryTasksResponse.Task\x1a\xce\x07\n\x04Task\x12\x11\n\ttask_name\x18\x02 \x02(\x0c\x12\x10\n\x08\x65ta_usec\x18\x03 \x02(\x03\x12\x0b\n\x03url\x18\x04 \x01(\x0c\x12P\n\x06method\x18\x05 \x01(\x0e\x32@.apphosting_bytes.TaskQueueQueryTasksResponse.Task.RequestMethod\x12\x13\n\x0bretry_count\x18\x06 \x01(\x05\x12I\n\x06header\x18\x07 \x03(\n29.apphosting_bytes.TaskQueueQueryTasksResponse.Task.Header\x12\x11\n\tbody_size\x18\n \x01(\x05\x12\x10\n\x04\x62ody\x18\x0b \x01(\x0c\x42\x02\x08\x01\x12\x1a\n\x12\x63reation_time_usec\x18\x0c \x02(\x03\x12W\n\rcrontimetable\x18\r \x01(\n2@.apphosting_bytes.TaskQueueQueryTasksResponse.Task.CronTimetable\x12I\n\x06runlog\x18\x10 \x01(\n29.apphosting_bytes.TaskQueueQueryTasksResponse.Task.RunLog\x12\x13\n\x0b\x64\x65scription\x18\x15 \x01(\x0c\x12.\n\x07payload\x18\x16 \x01(\x0b\x32\x1d.apphosting_bytes.TaskPayload\x12\x44\n\x10retry_parameters\x18\x17 \x01(\x0b\x32*.apphosting_bytes.TaskQueueRetryParameters\x12\x16\n\x0e\x66irst_try_usec\x18\x18 \x01(\x03\x12\x0b\n\x03tag\x18\x19 \x01(\x0c\x12\x17\n\x0f\x65xecution_count\x18\x1a \x01(\x05\x12\x1e\n\x16\x64ispatch_deadline_usec\x18\x1c \x01(\x03\x1a$\n\x06Header\x12\x0b\n\x03key\x18\x08 \x02(\x0c\x12\r\n\x05value\x18\t \x02(\x0c\x1a\x33\n\rCronTimetable\x12\x10\n\x08schedule\x18\x0e \x02(\x0c\x12\x10\n\x08timezone\x18\x0f \x02(\x0c\x1av\n\x06RunLog\x12\x17\n\x0f\x64ispatched_usec\x18\x11 \x02(\x03\x12\x10\n\x08lag_usec\x18\x12 \x02(\x03\x12\x14\n\x0c\x65lapsed_usec\x18\x13 \x02(\x03\x12\x15\n\rresponse_code\x18\x14 \x01(\x03\x12\x14\n\x0cretry_reason\x18\x1b \x01(\t\"A\n\rRequestMethod\x12\x07\n\x03GET\x10\x01\x12\x08\n\x04POST\x10\x02\x12\x08\n\x04HEAD\x10\x03\x12\x07\n\x03PUT\x10\x04\x12\n\n\x06\x44\x45LETE\x10\x05\"R\n\x19TaskQueueFetchTaskRequest\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\x0c\x12\x12\n\nqueue_name\x18\x02 \x02(\x0c\x12\x11\n\ttask_name\x18\x03 \x02(\x0c\"Y\n\x1aTaskQueueFetchTaskResponse\x12;\n\x04task\x18\x01 \x02(\x0b\x32-.apphosting_bytes.TaskQueueQueryTasksResponse\"C\n\"TaskQueueUpdateStorageLimitRequest\x12\x0e\n\x06\x61pp_id\x18\x01 \x02(\x0c\x12\r\n\x05limit\x18\x02 \x02(\x03\"8\n#TaskQueueUpdateStorageLimitResponse\x12\x11\n\tnew_limit\x18\x01 \x02(\x03\"\x83\x01\n TaskQueueQueryAndOwnTasksRequest\x12\x12\n\nqueue_name\x18\x01 \x02(\x0c\x12\x15\n\rlease_seconds\x18\x02 \x02(\x01\x12\x11\n\tmax_tasks\x18\x03 \x02(\x03\x12\x14\n\x0cgroup_by_tag\x18\x04 \x01(\x08\x12\x0b\n\x03tag\x18\x05 \x01(\x0c\"\xcc\x01\n!TaskQueueQueryAndOwnTasksResponse\x12\x46\n\x04task\x18\x01 \x03(\n28.apphosting_bytes.TaskQueueQueryAndOwnTasksResponse.Task\x1a_\n\x04Task\x12\x11\n\ttask_name\x18\x02 \x02(\x0c\x12\x10\n\x08\x65ta_usec\x18\x03 \x02(\x03\x12\x13\n\x0bretry_count\x18\x04 \x01(\x05\x12\x10\n\x04\x62ody\x18\x05 \x01(\x0c\x42\x02\x08\x01\x12\x0b\n\x03tag\x18\x06 \x01(\x0c\"q\n\x1fTaskQueueModifyTaskLeaseRequest\x12\x12\n\nqueue_name\x18\x01 \x02(\x0c\x12\x11\n\ttask_name\x18\x02 \x02(\x0c\x12\x10\n\x08\x65ta_usec\x18\x03 \x02(\x03\x12\x15\n\rlease_seconds\x18\x04 \x02(\x01\"<\n TaskQueueModifyTaskLeaseResponse\x12\x18\n\x10updated_eta_usec\x18\x01 \x02(\x03\x42\x31\n\"com.google.appengine.api.taskqueueB\x0bTaskQueuePb')



_TASKQUEUESERVICEERROR = DESCRIPTOR.message_types_by_name['TaskQueueServiceError']
_TASKPAYLOAD = DESCRIPTOR.message_types_by_name['TaskPayload']
_TASKQUEUERETRYPARAMETERS = DESCRIPTOR.message_types_by_name['TaskQueueRetryParameters']
_TASKQUEUEACL = DESCRIPTOR.message_types_by_name['TaskQueueAcl']
_TASKQUEUEHTTPHEADER = DESCRIPTOR.message_types_by_name['TaskQueueHttpHeader']
_TASKQUEUEMODE = DESCRIPTOR.message_types_by_name['TaskQueueMode']
_TASKQUEUEADDREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueAddRequest']
_TASKQUEUEADDREQUEST_HEADER = _TASKQUEUEADDREQUEST.nested_types_by_name['Header']
_TASKQUEUEADDREQUEST_CRONTIMETABLE = _TASKQUEUEADDREQUEST.nested_types_by_name['CronTimetable']
_TASKQUEUEADDRESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueAddResponse']
_TASKQUEUEBULKADDREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueBulkAddRequest']
_TASKQUEUEBULKADDRESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueBulkAddResponse']
_TASKQUEUEBULKADDRESPONSE_TASKRESULT = _TASKQUEUEBULKADDRESPONSE.nested_types_by_name['TaskResult']
_TASKQUEUEDELETEREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueDeleteRequest']
_TASKQUEUEDELETERESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueDeleteResponse']
_TASKQUEUEFORCERUNREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueForceRunRequest']
_TASKQUEUEFORCERUNRESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueForceRunResponse']
_TASKQUEUEUPDATEQUEUEREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueUpdateQueueRequest']
_TASKQUEUEUPDATEQUEUERESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueUpdateQueueResponse']
_TASKQUEUEFETCHQUEUESREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueFetchQueuesRequest']
_TASKQUEUEFETCHQUEUESRESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueFetchQueuesResponse']
_TASKQUEUEFETCHQUEUESRESPONSE_QUEUE = _TASKQUEUEFETCHQUEUESRESPONSE.nested_types_by_name['Queue']
_TASKQUEUEFETCHQUEUESTATSREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueFetchQueueStatsRequest']
_TASKQUEUESCANNERQUEUEINFO = DESCRIPTOR.message_types_by_name['TaskQueueScannerQueueInfo']
_TASKQUEUEFETCHQUEUESTATSRESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueFetchQueueStatsResponse']
_TASKQUEUEFETCHQUEUESTATSRESPONSE_QUEUESTATS = _TASKQUEUEFETCHQUEUESTATSRESPONSE.nested_types_by_name['QueueStats']
_TASKQUEUEPAUSEQUEUEREQUEST = DESCRIPTOR.message_types_by_name['TaskQueuePauseQueueRequest']
_TASKQUEUEPAUSEQUEUERESPONSE = DESCRIPTOR.message_types_by_name['TaskQueuePauseQueueResponse']
_TASKQUEUEPURGEQUEUEREQUEST = DESCRIPTOR.message_types_by_name['TaskQueuePurgeQueueRequest']
_TASKQUEUEPURGEQUEUERESPONSE = DESCRIPTOR.message_types_by_name['TaskQueuePurgeQueueResponse']
_TASKQUEUEDELETEQUEUEREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueDeleteQueueRequest']
_TASKQUEUEDELETEQUEUERESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueDeleteQueueResponse']
_TASKQUEUEDELETEGROUPREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueDeleteGroupRequest']
_TASKQUEUEDELETEGROUPRESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueDeleteGroupResponse']
_TASKQUEUEQUERYTASKSREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueQueryTasksRequest']
_TASKQUEUEQUERYTASKSRESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueQueryTasksResponse']
_TASKQUEUEQUERYTASKSRESPONSE_TASK = _TASKQUEUEQUERYTASKSRESPONSE.nested_types_by_name['Task']
_TASKQUEUEQUERYTASKSRESPONSE_TASK_HEADER = _TASKQUEUEQUERYTASKSRESPONSE_TASK.nested_types_by_name['Header']
_TASKQUEUEQUERYTASKSRESPONSE_TASK_CRONTIMETABLE = _TASKQUEUEQUERYTASKSRESPONSE_TASK.nested_types_by_name['CronTimetable']
_TASKQUEUEQUERYTASKSRESPONSE_TASK_RUNLOG = _TASKQUEUEQUERYTASKSRESPONSE_TASK.nested_types_by_name['RunLog']
_TASKQUEUEFETCHTASKREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueFetchTaskRequest']
_TASKQUEUEFETCHTASKRESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueFetchTaskResponse']
_TASKQUEUEUPDATESTORAGELIMITREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueUpdateStorageLimitRequest']
_TASKQUEUEUPDATESTORAGELIMITRESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueUpdateStorageLimitResponse']
_TASKQUEUEQUERYANDOWNTASKSREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueQueryAndOwnTasksRequest']
_TASKQUEUEQUERYANDOWNTASKSRESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueQueryAndOwnTasksResponse']
_TASKQUEUEQUERYANDOWNTASKSRESPONSE_TASK = _TASKQUEUEQUERYANDOWNTASKSRESPONSE.nested_types_by_name['Task']
_TASKQUEUEMODIFYTASKLEASEREQUEST = DESCRIPTOR.message_types_by_name['TaskQueueModifyTaskLeaseRequest']
_TASKQUEUEMODIFYTASKLEASERESPONSE = DESCRIPTOR.message_types_by_name['TaskQueueModifyTaskLeaseResponse']
_TASKQUEUESERVICEERROR_ERRORCODE = _TASKQUEUESERVICEERROR.enum_types_by_name['ErrorCode']
_TASKQUEUEMODE_MODE = _TASKQUEUEMODE.enum_types_by_name['Mode']
_TASKQUEUEADDREQUEST_REQUESTMETHOD = _TASKQUEUEADDREQUEST.enum_types_by_name['RequestMethod']
_TASKQUEUEQUERYTASKSRESPONSE_TASK_REQUESTMETHOD = _TASKQUEUEQUERYTASKSRESPONSE_TASK.enum_types_by_name['RequestMethod']
TaskQueueServiceError = _reflection.GeneratedProtocolMessageType('TaskQueueServiceError', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUESERVICEERROR,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueServiceError)

TaskPayload = _reflection.GeneratedProtocolMessageType('TaskPayload', (_message.Message,), {
  'DESCRIPTOR' : _TASKPAYLOAD,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskPayload)

TaskQueueRetryParameters = _reflection.GeneratedProtocolMessageType('TaskQueueRetryParameters', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUERETRYPARAMETERS,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueRetryParameters)

TaskQueueAcl = _reflection.GeneratedProtocolMessageType('TaskQueueAcl', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEACL,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueAcl)

TaskQueueHttpHeader = _reflection.GeneratedProtocolMessageType('TaskQueueHttpHeader', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEHTTPHEADER,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueHttpHeader)

TaskQueueMode = _reflection.GeneratedProtocolMessageType('TaskQueueMode', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEMODE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueMode)

TaskQueueAddRequest = _reflection.GeneratedProtocolMessageType('TaskQueueAddRequest', (_message.Message,), {

  'Header' : _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
    'DESCRIPTOR' : _TASKQUEUEADDREQUEST_HEADER,
    '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

    })
  ,

  'CronTimetable' : _reflection.GeneratedProtocolMessageType('CronTimetable', (_message.Message,), {
    'DESCRIPTOR' : _TASKQUEUEADDREQUEST_CRONTIMETABLE,
    '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

    })
  ,
  'DESCRIPTOR' : _TASKQUEUEADDREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueAddRequest)
_sym_db.RegisterMessage(TaskQueueAddRequest.Header)
_sym_db.RegisterMessage(TaskQueueAddRequest.CronTimetable)

TaskQueueAddResponse = _reflection.GeneratedProtocolMessageType('TaskQueueAddResponse', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEADDRESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueAddResponse)

TaskQueueBulkAddRequest = _reflection.GeneratedProtocolMessageType('TaskQueueBulkAddRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEBULKADDREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueBulkAddRequest)

TaskQueueBulkAddResponse = _reflection.GeneratedProtocolMessageType('TaskQueueBulkAddResponse', (_message.Message,), {

  'TaskResult' : _reflection.GeneratedProtocolMessageType('TaskResult', (_message.Message,), {
    'DESCRIPTOR' : _TASKQUEUEBULKADDRESPONSE_TASKRESULT,
    '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

    })
  ,
  'DESCRIPTOR' : _TASKQUEUEBULKADDRESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueBulkAddResponse)
_sym_db.RegisterMessage(TaskQueueBulkAddResponse.TaskResult)

TaskQueueDeleteRequest = _reflection.GeneratedProtocolMessageType('TaskQueueDeleteRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEDELETEREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueDeleteRequest)

TaskQueueDeleteResponse = _reflection.GeneratedProtocolMessageType('TaskQueueDeleteResponse', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEDELETERESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueDeleteResponse)

TaskQueueForceRunRequest = _reflection.GeneratedProtocolMessageType('TaskQueueForceRunRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEFORCERUNREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueForceRunRequest)

TaskQueueForceRunResponse = _reflection.GeneratedProtocolMessageType('TaskQueueForceRunResponse', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEFORCERUNRESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueForceRunResponse)

TaskQueueUpdateQueueRequest = _reflection.GeneratedProtocolMessageType('TaskQueueUpdateQueueRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEUPDATEQUEUEREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueUpdateQueueRequest)

TaskQueueUpdateQueueResponse = _reflection.GeneratedProtocolMessageType('TaskQueueUpdateQueueResponse', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEUPDATEQUEUERESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueUpdateQueueResponse)

TaskQueueFetchQueuesRequest = _reflection.GeneratedProtocolMessageType('TaskQueueFetchQueuesRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEFETCHQUEUESREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueFetchQueuesRequest)

TaskQueueFetchQueuesResponse = _reflection.GeneratedProtocolMessageType('TaskQueueFetchQueuesResponse', (_message.Message,), {

  'Queue' : _reflection.GeneratedProtocolMessageType('Queue', (_message.Message,), {
    'DESCRIPTOR' : _TASKQUEUEFETCHQUEUESRESPONSE_QUEUE,
    '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

    })
  ,
  'DESCRIPTOR' : _TASKQUEUEFETCHQUEUESRESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueFetchQueuesResponse)
_sym_db.RegisterMessage(TaskQueueFetchQueuesResponse.Queue)

TaskQueueFetchQueueStatsRequest = _reflection.GeneratedProtocolMessageType('TaskQueueFetchQueueStatsRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEFETCHQUEUESTATSREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueFetchQueueStatsRequest)

TaskQueueScannerQueueInfo = _reflection.GeneratedProtocolMessageType('TaskQueueScannerQueueInfo', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUESCANNERQUEUEINFO,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueScannerQueueInfo)

TaskQueueFetchQueueStatsResponse = _reflection.GeneratedProtocolMessageType('TaskQueueFetchQueueStatsResponse', (_message.Message,), {

  'QueueStats' : _reflection.GeneratedProtocolMessageType('QueueStats', (_message.Message,), {
    'DESCRIPTOR' : _TASKQUEUEFETCHQUEUESTATSRESPONSE_QUEUESTATS,
    '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

    })
  ,
  'DESCRIPTOR' : _TASKQUEUEFETCHQUEUESTATSRESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueFetchQueueStatsResponse)
_sym_db.RegisterMessage(TaskQueueFetchQueueStatsResponse.QueueStats)

TaskQueuePauseQueueRequest = _reflection.GeneratedProtocolMessageType('TaskQueuePauseQueueRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEPAUSEQUEUEREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueuePauseQueueRequest)

TaskQueuePauseQueueResponse = _reflection.GeneratedProtocolMessageType('TaskQueuePauseQueueResponse', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEPAUSEQUEUERESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueuePauseQueueResponse)

TaskQueuePurgeQueueRequest = _reflection.GeneratedProtocolMessageType('TaskQueuePurgeQueueRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEPURGEQUEUEREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueuePurgeQueueRequest)

TaskQueuePurgeQueueResponse = _reflection.GeneratedProtocolMessageType('TaskQueuePurgeQueueResponse', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEPURGEQUEUERESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueuePurgeQueueResponse)

TaskQueueDeleteQueueRequest = _reflection.GeneratedProtocolMessageType('TaskQueueDeleteQueueRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEDELETEQUEUEREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueDeleteQueueRequest)

TaskQueueDeleteQueueResponse = _reflection.GeneratedProtocolMessageType('TaskQueueDeleteQueueResponse', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEDELETEQUEUERESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueDeleteQueueResponse)

TaskQueueDeleteGroupRequest = _reflection.GeneratedProtocolMessageType('TaskQueueDeleteGroupRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEDELETEGROUPREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueDeleteGroupRequest)

TaskQueueDeleteGroupResponse = _reflection.GeneratedProtocolMessageType('TaskQueueDeleteGroupResponse', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEDELETEGROUPRESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueDeleteGroupResponse)

TaskQueueQueryTasksRequest = _reflection.GeneratedProtocolMessageType('TaskQueueQueryTasksRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEQUERYTASKSREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueQueryTasksRequest)

TaskQueueQueryTasksResponse = _reflection.GeneratedProtocolMessageType('TaskQueueQueryTasksResponse', (_message.Message,), {

  'Task' : _reflection.GeneratedProtocolMessageType('Task', (_message.Message,), {

    'Header' : _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
      'DESCRIPTOR' : _TASKQUEUEQUERYTASKSRESPONSE_TASK_HEADER,
      '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

      })
    ,

    'CronTimetable' : _reflection.GeneratedProtocolMessageType('CronTimetable', (_message.Message,), {
      'DESCRIPTOR' : _TASKQUEUEQUERYTASKSRESPONSE_TASK_CRONTIMETABLE,
      '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

      })
    ,

    'RunLog' : _reflection.GeneratedProtocolMessageType('RunLog', (_message.Message,), {
      'DESCRIPTOR' : _TASKQUEUEQUERYTASKSRESPONSE_TASK_RUNLOG,
      '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

      })
    ,
    'DESCRIPTOR' : _TASKQUEUEQUERYTASKSRESPONSE_TASK,
    '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

    })
  ,
  'DESCRIPTOR' : _TASKQUEUEQUERYTASKSRESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueQueryTasksResponse)
_sym_db.RegisterMessage(TaskQueueQueryTasksResponse.Task)
_sym_db.RegisterMessage(TaskQueueQueryTasksResponse.Task.Header)
_sym_db.RegisterMessage(TaskQueueQueryTasksResponse.Task.CronTimetable)
_sym_db.RegisterMessage(TaskQueueQueryTasksResponse.Task.RunLog)

TaskQueueFetchTaskRequest = _reflection.GeneratedProtocolMessageType('TaskQueueFetchTaskRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEFETCHTASKREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueFetchTaskRequest)

TaskQueueFetchTaskResponse = _reflection.GeneratedProtocolMessageType('TaskQueueFetchTaskResponse', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEFETCHTASKRESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueFetchTaskResponse)

TaskQueueUpdateStorageLimitRequest = _reflection.GeneratedProtocolMessageType('TaskQueueUpdateStorageLimitRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEUPDATESTORAGELIMITREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueUpdateStorageLimitRequest)

TaskQueueUpdateStorageLimitResponse = _reflection.GeneratedProtocolMessageType('TaskQueueUpdateStorageLimitResponse', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEUPDATESTORAGELIMITRESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueUpdateStorageLimitResponse)

TaskQueueQueryAndOwnTasksRequest = _reflection.GeneratedProtocolMessageType('TaskQueueQueryAndOwnTasksRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEQUERYANDOWNTASKSREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueQueryAndOwnTasksRequest)

TaskQueueQueryAndOwnTasksResponse = _reflection.GeneratedProtocolMessageType('TaskQueueQueryAndOwnTasksResponse', (_message.Message,), {

  'Task' : _reflection.GeneratedProtocolMessageType('Task', (_message.Message,), {
    'DESCRIPTOR' : _TASKQUEUEQUERYANDOWNTASKSRESPONSE_TASK,
    '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

    })
  ,
  'DESCRIPTOR' : _TASKQUEUEQUERYANDOWNTASKSRESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueQueryAndOwnTasksResponse)
_sym_db.RegisterMessage(TaskQueueQueryAndOwnTasksResponse.Task)

TaskQueueModifyTaskLeaseRequest = _reflection.GeneratedProtocolMessageType('TaskQueueModifyTaskLeaseRequest', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEMODIFYTASKLEASEREQUEST,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueModifyTaskLeaseRequest)

TaskQueueModifyTaskLeaseResponse = _reflection.GeneratedProtocolMessageType('TaskQueueModifyTaskLeaseResponse', (_message.Message,), {
  'DESCRIPTOR' : _TASKQUEUEMODIFYTASKLEASERESPONSE,
  '__module__' : 'google.appengine.api.taskqueue.taskqueue_service_bytes_pb2'

  })
_sym_db.RegisterMessage(TaskQueueModifyTaskLeaseResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\"com.google.appengine.api.taskqueueB\013TaskQueuePb'
  _TASKPAYLOAD._options = None
  _TASKPAYLOAD._serialized_options = b'\010\001'
  _TASKQUEUEADDREQUEST.fields_by_name['body']._options = None
  _TASKQUEUEADDREQUEST.fields_by_name['body']._serialized_options = b'\010\001'
  _TASKQUEUEFETCHQUEUESRESPONSE_QUEUE.fields_by_name['creator_name']._options = None
  _TASKQUEUEFETCHQUEUESRESPONSE_QUEUE.fields_by_name['creator_name']._serialized_options = b'\010\001'
  _TASKQUEUEQUERYTASKSRESPONSE_TASK.fields_by_name['body']._options = None
  _TASKQUEUEQUERYTASKSRESPONSE_TASK.fields_by_name['body']._serialized_options = b'\010\001'
  _TASKQUEUEQUERYANDOWNTASKSRESPONSE_TASK.fields_by_name['body']._options = None
  _TASKQUEUEQUERYANDOWNTASKSRESPONSE_TASK.fields_by_name['body']._serialized_options = b'\010\001'
  _TASKQUEUESERVICEERROR._serialized_start=136
  _TASKQUEUESERVICEERROR._serialized_end=1448
  _TASKQUEUESERVICEERROR_ERRORCODE._serialized_start=162
  _TASKQUEUESERVICEERROR_ERRORCODE._serialized_end=1448
  _TASKPAYLOAD._serialized_start=1450
  _TASKPAYLOAD._serialized_end=1477
  _TASKQUEUERETRYPARAMETERS._serialized_start=1480
  _TASKQUEUERETRYPARAMETERS._serialized_end=1638
  _TASKQUEUEACL._serialized_start=1640
  _TASKQUEUEACL._serialized_end=1696
  _TASKQUEUEHTTPHEADER._serialized_start=1698
  _TASKQUEUEHTTPHEADER._serialized_end=1747
  _TASKQUEUEMODE._serialized_start=1749
  _TASKQUEUEMODE._serialized_end=1792
  _TASKQUEUEMODE_MODE._serialized_start=1766
  _TASKQUEUEMODE_MODE._serialized_end=1792
  _TASKQUEUEADDREQUEST._serialized_start=1795
  _TASKQUEUEADDREQUEST._serialized_end=2698
  _TASKQUEUEADDREQUEST_HEADER._serialized_start=2542
  _TASKQUEUEADDREQUEST_HEADER._serialized_end=2578
  _TASKQUEUEADDREQUEST_CRONTIMETABLE._serialized_start=2580
  _TASKQUEUEADDREQUEST_CRONTIMETABLE._serialized_end=2631
  _TASKQUEUEADDREQUEST_REQUESTMETHOD._serialized_start=2633
  _TASKQUEUEADDREQUEST_REQUESTMETHOD._serialized_end=2698
  _TASKQUEUEADDRESPONSE._serialized_start=2700
  _TASKQUEUEADDRESPONSE._serialized_end=2748
  _TASKQUEUEBULKADDREQUEST._serialized_start=2750
  _TASKQUEUEBULKADDREQUEST._serialized_end=2835
  _TASKQUEUEBULKADDRESPONSE._serialized_start=2838
  _TASKQUEUEBULKADDRESPONSE._serialized_end=3046
  _TASKQUEUEBULKADDRESPONSE_TASKRESULT._serialized_start=2941
  _TASKQUEUEBULKADDRESPONSE_TASKRESULT._serialized_end=3046
  _TASKQUEUEDELETEREQUEST._serialized_start=3048
  _TASKQUEUEDELETEREQUEST._serialized_end=3127
  _TASKQUEUEDELETERESPONSE._serialized_start=3129
  _TASKQUEUEDELETERESPONSE._serialized_end=3221
  _TASKQUEUEFORCERUNREQUEST._serialized_start=3223
  _TASKQUEUEFORCERUNREQUEST._serialized_end=3304
  _TASKQUEUEFORCERUNRESPONSE._serialized_start=3306
  _TASKQUEUEFORCERUNRESPONSE._serialized_end=3400
  _TASKQUEUEUPDATEQUEUEREQUEST._serialized_start=3403
  _TASKQUEUEUPDATEQUEUEREQUEST._serialized_end=3820
  _TASKQUEUEUPDATEQUEUERESPONSE._serialized_start=3822
  _TASKQUEUEUPDATEQUEUERESPONSE._serialized_end=3852
  _TASKQUEUEFETCHQUEUESREQUEST._serialized_start=3854
  _TASKQUEUEFETCHQUEUESREQUEST._serialized_end=3917
  _TASKQUEUEFETCHQUEUESRESPONSE._serialized_start=3920
  _TASKQUEUEFETCHQUEUESRESPONSE._serialized_end=4455
  _TASKQUEUEFETCHQUEUESRESPONSE_QUEUE._serialized_start=4022
  _TASKQUEUEFETCHQUEUESRESPONSE_QUEUE._serialized_end=4455
  _TASKQUEUEFETCHQUEUESTATSREQUEST._serialized_start=4457
  _TASKQUEUEFETCHQUEUESTATSREQUEST._serialized_end=4549
  _TASKQUEUESCANNERQUEUEINFO._serialized_start=4552
  _TASKQUEUESCANNERQUEUEINFO._serialized_end=4723
  _TASKQUEUEFETCHQUEUESTATSRESPONSE._serialized_start=4726
  _TASKQUEUEFETCHQUEUESTATSRESPONSE._serialized_end=4968
  _TASKQUEUEFETCHQUEUESTATSRESPONSE_QUEUESTATS._serialized_start=4845
  _TASKQUEUEFETCHQUEUESTATSRESPONSE_QUEUESTATS._serialized_end=4968
  _TASKQUEUEPAUSEQUEUEREQUEST._serialized_start=4970
  _TASKQUEUEPAUSEQUEUEREQUEST._serialized_end=5049
  _TASKQUEUEPAUSEQUEUERESPONSE._serialized_start=5051
  _TASKQUEUEPAUSEQUEUERESPONSE._serialized_end=5080
  _TASKQUEUEPURGEQUEUEREQUEST._serialized_start=5082
  _TASKQUEUEPURGEQUEUEREQUEST._serialized_end=5146
  _TASKQUEUEPURGEQUEUERESPONSE._serialized_start=5148
  _TASKQUEUEPURGEQUEUERESPONSE._serialized_end=5177
  _TASKQUEUEDELETEQUEUEREQUEST._serialized_start=5179
  _TASKQUEUEDELETEQUEUEREQUEST._serialized_end=5244
  _TASKQUEUEDELETEQUEUERESPONSE._serialized_start=5246
  _TASKQUEUEDELETEQUEUERESPONSE._serialized_end=5276
  _TASKQUEUEDELETEGROUPREQUEST._serialized_start=5278
  _TASKQUEUEDELETEGROUPREQUEST._serialized_end=5323
  _TASKQUEUEDELETEGROUPRESPONSE._serialized_start=5325
  _TASKQUEUEDELETEGROUPRESPONSE._serialized_end=5355
  _TASKQUEUEQUERYTASKSREQUEST._serialized_start=5358
  _TASKQUEUEQUERYTASKSREQUEST._serialized_end=5511
  _TASKQUEUEQUERYTASKSRESPONSE._serialized_start=5514
  _TASKQUEUEQUERYTASKSRESPONSE._serialized_end=6586
  _TASKQUEUEQUERYTASKSRESPONSE_TASK._serialized_start=5612
  _TASKQUEUEQUERYTASKSRESPONSE_TASK._serialized_end=6586
  _TASKQUEUEQUERYTASKSRESPONSE_TASK_HEADER._serialized_start=6310
  _TASKQUEUEQUERYTASKSRESPONSE_TASK_HEADER._serialized_end=6346
  _TASKQUEUEQUERYTASKSRESPONSE_TASK_CRONTIMETABLE._serialized_start=6348
  _TASKQUEUEQUERYTASKSRESPONSE_TASK_CRONTIMETABLE._serialized_end=6399
  _TASKQUEUEQUERYTASKSRESPONSE_TASK_RUNLOG._serialized_start=6401
  _TASKQUEUEQUERYTASKSRESPONSE_TASK_RUNLOG._serialized_end=6519
  _TASKQUEUEQUERYTASKSRESPONSE_TASK_REQUESTMETHOD._serialized_start=2633
  _TASKQUEUEQUERYTASKSRESPONSE_TASK_REQUESTMETHOD._serialized_end=2698
  _TASKQUEUEFETCHTASKREQUEST._serialized_start=6588
  _TASKQUEUEFETCHTASKREQUEST._serialized_end=6670
  _TASKQUEUEFETCHTASKRESPONSE._serialized_start=6672
  _TASKQUEUEFETCHTASKRESPONSE._serialized_end=6761
  _TASKQUEUEUPDATESTORAGELIMITREQUEST._serialized_start=6763
  _TASKQUEUEUPDATESTORAGELIMITREQUEST._serialized_end=6830
  _TASKQUEUEUPDATESTORAGELIMITRESPONSE._serialized_start=6832
  _TASKQUEUEUPDATESTORAGELIMITRESPONSE._serialized_end=6888
  _TASKQUEUEQUERYANDOWNTASKSREQUEST._serialized_start=6891
  _TASKQUEUEQUERYANDOWNTASKSREQUEST._serialized_end=7022
  _TASKQUEUEQUERYANDOWNTASKSRESPONSE._serialized_start=7025
  _TASKQUEUEQUERYANDOWNTASKSRESPONSE._serialized_end=7229
  _TASKQUEUEQUERYANDOWNTASKSRESPONSE_TASK._serialized_start=7134
  _TASKQUEUEQUERYANDOWNTASKSRESPONSE_TASK._serialized_end=7229
  _TASKQUEUEMODIFYTASKLEASEREQUEST._serialized_start=7231
  _TASKQUEUEMODIFYTASKLEASEREQUEST._serialized_end=7344
  _TASKQUEUEMODIFYTASKLEASERESPONSE._serialized_start=7346
  _TASKQUEUEMODIFYTASKLEASERESPONSE._serialized_end=7406




#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""A module to use service stubs for testing.

To test applications that use App Engine services, such as datastore,
developers can use the available stub implementations. Service stubs behave like
the original service without causing permanent side effects. The datastore stub,
for example, allows you to write entities into memory without storing them to
the actual datastore. The `testbed` module makes using those stubs for testing
easier.

Example:

    import unittest

    from google.appengine.ext import ndb
    from google.appengine.ext import testbed


    class TestModel(ndb.Model):
      number = ndb.IntegerProperty(default=42)


    class MyTestCase(unittest.TestCase):

      def setUp(self):
        # First, create an instance of the Testbed class.
        self.testbed = testbed.Testbed()
        # Then activate the testbed, which will allow you to use
        # service stubs.
        self.testbed.activate()
        # Next, declare which service stubs you want to use.
        self.testbed.init_datastore_v3_stub()
        self.testbed.init_memcache_stub()

      def tearDown(self):
        # Don't forget to deactivate the testbed after the tests are
        # completed. If the testbed is not deactivated, the original
        # stubs will not be restored.
        self.testbed.deactivate()

      def testInsertEntity(self):
        # Because we use the datastore stub, this put() does not have
        # permanent side effects.
        TestModel().put()
        fetched_entities = TestModel.query().fetch(2)
        self.assertEqual(1, len(fetched_entities))
        self.assertEqual(42, fetched_entities[0].number)



Enable stubs and disable services
---------------------------------

The `testbed` module allows you to use stubs for the following services:

    - capability_service
    - channel
    - datastore_v3 (aka datastore)
    - images (only for dev_appserver)
    - mail (only for dev_appserver)
    - memcache
    - taskqueue
    - urlfetch
    - user
    - xmpp

To use a particular service stub, call `self.init_SERVICENAME_stub()`. Using
this call will replace calls to the service with calls to the service stub. If
you want to disable any calls to a particular service, call
`self.init_SERVICENAME_stub(enable=False)`. This call can be useful if you want
to test code that must not use a certain service.


Environment variables
---------------------

App Engine service stubs often depend on environment variables. For example, the
datastore stub uses an environment variable to store entities linked to a
particular app. `testbed` will use default values if nothing else is provided,
but you can change those values with `self.setup_env()`.
"""








import contextvars
import os
import re
import sys
import threading





import attr
from google.appengine.api import apiproxy_stub
from google.appengine.api import apiproxy_stub_map
from google.appengine.api import datastore_file_stub
from google.appengine.api import full_app_id
from google.appengine.api import request_info
from google.appengine.api import stublib
from google.appengine.api import urlfetch_stub
from google.appengine.api import user_service_stub
from google.appengine.api.app_identity import app_identity
from google.appengine.api.app_identity import app_identity_stub
from google.appengine.api.blobstore import blobstore_stub
from google.appengine.api.blobstore import dict_blob_storage
from google.appengine.api.capabilities import capability_stub
from google.appengine.api.images import images_stub
from google.appengine.api.memcache import memcache_stub
from google.appengine.api.modules import modules_stub
from google.appengine.api.namespace_manager import namespace_manager
from google.appengine.api.oauth import oauth_api
from google.appengine.api.taskqueue import taskqueue_stub
from google.appengine.datastore import cloud_datastore_v1_stub
from google.appengine.datastore import datastore_pbs
from google.appengine.datastore import datastore_stub_util
from google.appengine.datastore import datastore_v4_stub
from google.appengine.runtime import context
import six








try:
  from google.appengine.api import mail_stub
except AttributeError:



  mail_stub = None






try:


  from google.appengine.ext.remote_api import remote_api_stub
except ImportError:
  pass
try:




  from google.appengine.ext.testbed import apiserver_util
except ImportError:
  apiserver_util = None


DEFAULT_ENVIRONMENT = {
    'GAE_APPLICATION': 'testbed-test',
    'GOOGLE_CLOUD_PROJECT': 'testbed-test',
    'AUTH_DOMAIN': 'gmail.com',
    'HTTP_HOST': 'testbed.example.com',
    'CURRENT_MODULE_ID': 'default',
    'CURRENT_VERSION_ID': 'testbed-version',
    'GAE_RUNTIME': 'python3' + str(sys.version_info.minor),
    'REQUEST_ID_HASH': 'testbed-request-id-hash',
    'REQUEST_LOG_ID': '7357B3D7091D',
    'SERVER_NAME': 'testbed.example.com',
    'SERVER_SOFTWARE': 'Development/1.0 (testbed)',
    'SERVER_PORT': '80',
    'USER_EMAIL': '',
    'USER_ID': '',
}

# Deprecated legacy aliases for default environment variables. New code

DEFAULT_APP_ID = DEFAULT_ENVIRONMENT['GAE_APPLICATION']
DEFAULT_AUTH_DOMAIN = DEFAULT_ENVIRONMENT['AUTH_DOMAIN']
DEFAULT_SERVER_NAME = DEFAULT_ENVIRONMENT['SERVER_NAME']
DEFAULT_SERVER_SOFTWARE = DEFAULT_ENVIRONMENT['SERVER_SOFTWARE']
DEFAULT_SERVER_PORT = DEFAULT_ENVIRONMENT['SERVER_PORT']


APP_IDENTITY_SERVICE_NAME = 'app_identity_service'
BLOBSTORE_SERVICE_NAME = 'blobstore'
CAPABILITY_SERVICE_NAME = 'capability_service'
CHANNEL_SERVICE_NAME = 'channel'
DATASTORE_SERVICE_NAME = 'datastore_v3'
FILES_SERVICE_NAME = 'file'
IMAGES_SERVICE_NAME = 'images'
MAIL_SERVICE_NAME = 'mail'
MEMCACHE_SERVICE_NAME = 'memcache'
TASKQUEUE_SERVICE_NAME = 'taskqueue'
URLFETCH_SERVICE_NAME = 'urlfetch'
USER_SERVICE_NAME = 'user'
XMPP_SERVICE_NAME = 'xmpp'
MODULES_SERVICE_NAME = 'modules'


INIT_STUB_METHOD_NAMES = {
    APP_IDENTITY_SERVICE_NAME: 'init_app_identity_stub',
    BLOBSTORE_SERVICE_NAME: 'init_blobstore_stub',
    CAPABILITY_SERVICE_NAME: 'init_capability_stub',
    DATASTORE_SERVICE_NAME: 'init_datastore_v3_stub',
    IMAGES_SERVICE_NAME: 'init_images_stub',
    MAIL_SERVICE_NAME: 'init_mail_stub',
    MEMCACHE_SERVICE_NAME: 'init_memcache_stub',
    MODULES_SERVICE_NAME: 'init_modules_stub',
    TASKQUEUE_SERVICE_NAME: 'init_taskqueue_stub',
    URLFETCH_SERVICE_NAME: 'init_urlfetch_stub',
    USER_SERVICE_NAME: 'init_user_stub',
}


SUPPORTED_SERVICES = sorted(INIT_STUB_METHOD_NAMES)


URLMATCHERS_TO_FETCH_FUNCTIONS = []


AUTO_ID_POLICY_SEQUENTIAL = datastore_stub_util.SEQUENTIAL
AUTO_ID_POLICY_SCATTERED = datastore_stub_util.SCATTERED


class Error(Exception):
  """Base testbed error type."""


class NotActivatedError(Error):
  """Raised if the used testbed instance is not activated."""


class StubNotSupportedError(Error):
  """Raised if an unsupported service stub is accessed."""


class EmulatorSupportChecker(object):
  """A static class. Checks whether datastore emulator is supported.














  """

  _check_lock = threading.Lock()
  _use_datastore_emulator = None
  _api_port = None

  @classmethod
  def get_api_port(cls):
    """Returns the integer port number that api_server listens on."""
    return cls._api_port

  @classmethod
  def get_emulator_port(cls):
    """Returns the integer port number that datastore emulator listens on."""
    return cls._emulator_port

  @classmethod
  def init(cls, api_port, emulator_port):
    cls._api_port = api_port
    cls._emulator_port = emulator_port
    cls._use_datastore_emulator = True

  @classmethod
  def check(cls):
    """Checks whether cloud datastore should be used.

    In a unit test process, the first call to this method sets the value of
    `_use_datastore_emulator` and `_api_port`. Subsequent calls to this method
    can read `_use_datastore_emulator`.

    Returns:
      A boolean that indicates whether cloud datastore should be used.
    """

    cls._check_lock.acquire()
    if cls._use_datastore_emulator is None:
      if os.environ.get('APPENGINE_TESTBED_USES_DATASTORE_EMULATOR'):

        cls._use_datastore_emulator = True
        cls._api_port = int(os.environ['API_SERVER_PORT'])
        cls._emulator_port = int(os.environ['DATASTORE_EMULATOR_PORT'])








      else:
        cls._use_datastore_emulator = False
    cls._check_lock.release()
    return cls._use_datastore_emulator


@attr.s
class ActiveStub(object):
  stub = attr.ib()
  deactivate_callback = attr.ib(default=None)



class Testbed(object):
  """Class providing APIs to manipulate stubs for testing.

  This class allows you to replace App Engine services with fake stub
  implementations. These stubs act like the actual APIs but do not invoke the
  replaced services.

  In order to use a fake service stub or disable a real service, invoke the
  corresponding `init_*_stub` methods of this class.
  """

  def __init__(self):
    self._activated = False

    self._enabled_stubs = {}

    self._blob_storage = None
    self._context_reset_tokens = {}

  def activate(self, use_datastore_emulator=False):
    """Activates the testbed.

    Invoking this method will also assign default values to environment
    variables that are required by App Engine services, such as
    the application ID. You can set custom values with `setup_env()`.

    Args:
      use_datastore_emulator: `True` if user specifies testbed to use the Cloud
        Datastore Emulator.
    """
    namespace_manager._TESTBED_RESET_TOKEN = None
    oauth_api._TESTBED_RESET_TOKENS.clear()
    self._context_reset_tokens = {}
    self._orig_env = dict(os.environ)
    self.setup_env()






    self._original_stub_map = apiproxy_stub_map.apiproxy
    self._test_stub_map = apiproxy_stub_map.APIProxyStubMap()
    internal_map = self._original_stub_map._APIProxyStubMap__stub_map
    self._test_stub_map._APIProxyStubMap__stub_map = dict(internal_map)
    apiproxy_stub_map.apiproxy = self._test_stub_map
    self._activated = True

    if use_datastore_emulator:
      if not EmulatorSupportChecker.check():
        api_port, emulator_port = apiserver_util.setup_api_server()
        EmulatorSupportChecker.init(api_port, emulator_port)
      self._use_datastore_emulator = True
    else:
      self._use_datastore_emulator = EmulatorSupportChecker.check()
    if self._use_datastore_emulator:
      self.api_port = EmulatorSupportChecker.get_api_port()
      self.rpc_server = remote_api_stub.ConfigureRemoteApi(
          full_app_id.get(),
          '/',
          lambda: ('', ''),
          'localhost:%d' % self.api_port,
          services=[],
          apiproxy=self._test_stub_map,
          use_remote_datastore=False)
      self._emulator_port = EmulatorSupportChecker.get_emulator_port()

  def deactivate(self):
    """Deactivates the testbed.

    This method will restore the API proxy and environment variables to the
    state they were in before `activate()` was called.

    Raises:
      NotActivatedError: If called before `activate()` was called.
    """
    if not self._activated:
      raise NotActivatedError('The testbed is not activated.')

    for _, active_stub in six.iteritems(self._enabled_stubs):
      if active_stub.deactivate_callback:
        active_stub.deactivate_callback(active_stub.stub)

    apiproxy_stub_map.apiproxy = self._original_stub_map
    self._enabled_stubs = {}


    os.environ.clear()
    os.environ.update(self._orig_env)

    all_reset_tokens = {
        **self._context_reset_tokens,
        **oauth_api._TESTBED_RESET_TOKENS,
        namespace_manager._CURRENT_NAMESPACE:
            namespace_manager._TESTBED_RESET_TOKEN,
    }

    for ctxvar, token in all_reset_tokens.items():
      if token:
        try:
          ctxvar.reset(token)
        except ValueError:




          pass

    namespace_manager._TESTBED_RESET_TOKEN = None
    oauth_api._TESTBED_RESET_TOKENS.clear()

    self._blob_storage = None
    self._activated = False

  def setup_env(self, overwrite=False, **kwargs):
    """Sets default and custom environment variables.

    By default, all of the items in `DEFAULT_ENVIRONMENT` will be created
    without being specified. To set a value other than the default, or to pass
    a custom environment variable, pass a corresponding keyword argument.

    Example:

        # All defaults
        testbed_instance.setup_env()
        # All defaults, overriding AUTH_DOMAIN in both context modes
        testbed_instance.setup_env(auth_domain='custom')
        # All defaults; adds a custom os.environ['CUSTOM'] = 'foo'
        testbed_instance.setup_env(custom='foo')


    To overwrite the values set by a previous invocation, pass `overwrite=True`.
    Passing this value will not result in an `OVERWRITE` entry in `os.environ`.

    If the variable corresponds to a WSGI environ variable, it will be set in
    the gen2 request context as contextvars as well as `os.environ`. To only set
    the gen2 context, switch to `setup_wsgi_env()`.

    Args:
      overwrite: Boolean. Specifies whether to overwrite items with
          corresponding entries in `os.environ`.
      **kwargs: Environment variables to set. The name of the argument will be
          uppercased and used as a key in `os.environ`.
    """
    user_specified = {key.upper(): value for key, value in kwargs.items()}








    specified_app_id = (user_specified.get('APP_ID')
                        or full_app_id.get(environ=user_specified)
                        or user_specified.get('GOOGLE_CLOUD_PROJECT'))
    full_app_id.clear(environ=user_specified)
    user_specified.pop('APP_ID', None)
    if specified_app_id:

      user_specified.setdefault('GAE_APPLICATION', specified_app_id)


      user_specified.setdefault('GOOGLE_CLOUD_PROJECT',
                                full_app_id.project_id(specified_app_id))

    merged_vars = user_specified.copy()
    if not overwrite:
      for key, value in six.iteritems(DEFAULT_ENVIRONMENT):
        if key not in merged_vars:
          merged_vars[key] = value

    ctx = contextvars.copy_context()
    gae_vars = {k: v for k, v in vars(context.gae_headers).items()
                if isinstance(v, contextvars.ContextVar)}
    wsgi_vars = {k: v for k, v in vars(context.wsgi).items()
                 if isinstance(v, contextvars.ContextVar)}
    oauth_vars = {
        v.name: v
        for k, v in vars(oauth_api).items()
        if isinstance(v, contextvars.ContextVar)
    }
    for key, value in six.iteritems(merged_vars):
      if key == 'GAE_APPLICATION':
        if overwrite or not full_app_id.get():
          full_app_id.put(value)
      elif oauth_vars.get(key):
        ctxvar = oauth_vars.get(key)
        if ctxvar.get(None) and overwrite:
          ctxvar.set(value)
        elif ctxvar.get(None) is None:
          token = ctxvar.set(value)
          oauth_api._TESTBED_RESET_TOKENS[ctxvar] = token
      elif overwrite or key not in os.environ:
        if key == 'GOOGLE_CLOUD_PROJECT':
          validate_project_id(value)
        os.environ[key] = value
        if key.startswith(context.gae_headers.PREFIX):

          key = key[len(context.gae_headers.PREFIX):]
        ctxvar = gae_vars.get(key, wsgi_vars.get(key))
        if ctxvar is not None and (overwrite or ctxvar not in ctx):
          wsgi_key = (context.gae_headers.PREFIX + key
                      if key in gae_vars
                      else key)
          self.setup_wsgi_env(**{wsgi_key: value})




  def setup_wsgi_env(self, **kwargs):
    """Sets the gen2 request context with the WSGI vars provided in kwargs.

    Valid values can be found in `google.appengine.runtime.context.gae_headers`
    and `google.appengine.runtime.context.wsgi`.

    setup_wsgi_env does not also set the "legacy context mode" aka `os.environ`.
    If you still rely on the legacy context, use setup_env instead.

    Example:
      # Sets context.gae_headers.AUTH_DOMAIN
      # Note that all as per PEP 3333, all headers have the HTTP_ prefix.
      # GAE headers headers share an additional x_appengine_ prefix.
      testbed_instance.setup_wsgi_env(http_x_appengine_auth_domain='custom')
      # Sets context.wsgi.SERVER_NAME
      testbed_instance.setup_wsgi_env(server_name='custom')

    Args:
      **kwargs: WSGI variables to set in the SDK's request context.
    """
    wsgi_env = {key.upper(): value for key, value in kwargs.items()}
    new_reset_tokens = context.init_from_wsgi_environ(wsgi_env).items()




    for key, value in new_reset_tokens:
      if key not in self._context_reset_tokens:
        self._context_reset_tokens[key] = value

  def _register_stub(self, service_name, stub, deactivate_callback=None):
    """Registers a service stub.

    Args:
      service_name: The name of the service the stub represents.
      stub: The stub.
      deactivate_callback: An optional function to call to deactivate the
          stub. Must accept the stub as the only argument.

    Raises:
      NotActivatedError: The testbed is not activated.
    """
    self._disable_stub(service_name)
    if isinstance(stub, stublib.Stub):
      stub.patchers.StartAll()
    if isinstance(stub, apiproxy_stub.APIProxyStub):
      self._test_stub_map.RegisterStub(service_name, stub)

    self._enabled_stubs[service_name] = ActiveStub(stub, deactivate_callback)

  def _disable_stub(self, service_name):
    """Disables a service stub.

    Args:
      service_name: The name of the service to disable.

    Raises:
      NotActivatedError: The testbed is not activated.
    """
    if not self._activated:
      raise NotActivatedError('The testbed is not activated.')
    if service_name in self._enabled_stubs:
      active_stub = self._enabled_stubs.pop(service_name)
      if active_stub.deactivate_callback:
        active_stub.deactivate_callback(active_stub.stub)
      if isinstance(active_stub.stub, stublib.Stub):
        active_stub.stub.patchers.StopAll()
    if service_name in self._test_stub_map._APIProxyStubMap__stub_map:
      del self._test_stub_map._APIProxyStubMap__stub_map[service_name]

  def get_stub(self, service_name):
    """Gets the stub for a service.

    Args:
      service_name: The name of the service.

    Returns:
      The stub for `service_name`.

    Raises:
      NotActivatedError: The testbed is not activated.
      StubNotSupportedError: The service is not supported by testbed.
      StubNotEnabledError: The service stub has not been enabled.
    """
    if not self._activated:
      raise NotActivatedError('The testbed is not activated.')
    if service_name not in SUPPORTED_SERVICES:
      msg = 'The "%s" service is not supported by testbed' % service_name
      raise StubNotSupportedError(msg)
    if service_name not in self._enabled_stubs:
      return None
    return self._enabled_stubs[service_name].stub

  def init_app_identity_stub(self, enable=True):
    """Enables the app identity stub.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.
    """
    if not enable:
      self._disable_stub(APP_IDENTITY_SERVICE_NAME)
      return

    stub = app_identity_stub.AppIdentityServiceStub()
    self._register_stub(APP_IDENTITY_SERVICE_NAME, stub)

  def _get_blob_storage(self):
    """Creates a blob storage for stubs if needed."""
    if self._blob_storage is None:
      self._blob_storage = dict_blob_storage.DictBlobStorage()
    return self._blob_storage

  def init_blobstore_stub(self, enable=True):
    """Enables the blobstore stub.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.
    """
    if not enable:
      self._disable_stub(BLOBSTORE_SERVICE_NAME)
      return

    stub = blobstore_stub.BlobstoreServiceStub(self._get_blob_storage())
    self._register_stub(BLOBSTORE_SERVICE_NAME, stub)

  def init_capability_stub(self, enable=True):
    """Enables the capability stub.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.
    """
    if not enable:
      self._disable_stub(CAPABILITY_SERVICE_NAME)
      return
    stub = capability_stub.CapabilityServiceStub()
    self._register_stub(CAPABILITY_SERVICE_NAME, stub)

  def init_channel_stub(self, enable=True):
    """Enables the channel stub.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.

    Raises:
      StubNotSupportedError: If called.
    """
    raise StubNotSupportedError(
        'The channel stub is not supported in Titanoboa.')

  def init_datastore_v3_stub(self, enable=True, datastore_file=None,
                             use_sqlite=False,
                             auto_id_policy=AUTO_ID_POLICY_SEQUENTIAL,
                             **stub_kw_args):
    """Enables the datastore stub.

    The `datastore_file` argument can be set to the path of an existing
    datastore file, or `None` (default) to use an in-memory datastore that is
    initially empty. If you use the sqlite stub and have defined
    `datastore_file`, the changes that you apply in a test will be written to
    the file. If you use the default datastore stub, changes are *not* saved to
    disk unless you set `save_changes=True`.

    Note:
        You can only access those entities of the datastore file that use the
        same application ID as the test run. You can change the application ID
        for a test with `setup_env()`.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.
      datastore_file: File name of a `dev_appserver` datastore file.
      use_sqlite: `True` to use the Sqlite stub, or `False` (default) to use
          the file stub.
      auto_id_policy: How datastore stub assigns auto IDs. This value can be
          either `AUTO_ID_POLICY_SEQUENTIAL` or `AUTO_ID_POLICY_SCATTERED`.
      **stub_kw_args: Keyword arguments passed on to the service stub.

    Raises:
      StubNotSupportedError: If `datastore_sqlite_stub` is `None`.
    """
    if self._use_datastore_emulator:





      self._disable_stub(DATASTORE_SERVICE_NAME)

      delegate_stub = (
          remote_api_stub.DatastoreStubTestbedDelegate(
              self.rpc_server, '/', stub_kw_args.get(
                  'max_request_size', apiproxy_stub.MAX_REQUEST_SIZE),
              emulator_port=self._emulator_port))
      delegate_stub.Clear()
      self._test_stub_map.RegisterStub(
          DATASTORE_SERVICE_NAME, delegate_stub)
      consistency_policy = stub_kw_args.get(
          'consistency_policy',
          datastore_stub_util.PseudoRandomHRConsistencyPolicy(probability=1.0))
      datastore_stub_util.UpdateEmulatorConfig(
          self._emulator_port, auto_id_policy, consistency_policy)
      if isinstance(consistency_policy,
                    datastore_stub_util.PseudoRandomHRConsistencyPolicy):
        consistency_policy.is_using_cloud_datastore_emulator = True
        consistency_policy.emulator_port = self._emulator_port


      self._enabled_stubs[DATASTORE_SERVICE_NAME] = ActiveStub(delegate_stub)
      return
    if not enable:
      self._disable_stub(DATASTORE_SERVICE_NAME)
      self._disable_stub(datastore_v4_stub.SERVICE_NAME)
      self._disable_stub(cloud_datastore_v1_stub.SERVICE_NAME)
      return
    if use_sqlite:











      raise NotImplementedError('datastore_sqlite_stub not supported')
    else:
      stub_kw_args.setdefault('save_changes', False)
      stub = datastore_file_stub.DatastoreFileStub(
          full_app_id.get(),
          datastore_file,
          use_atexit=False,
          auto_id_policy=auto_id_policy,
          **stub_kw_args)
    self._register_stub(DATASTORE_SERVICE_NAME, stub,
                        self._deactivate_datastore_v3_stub)
    v4_stub = datastore_v4_stub.DatastoreV4Stub(full_app_id.get())
    self._register_stub(datastore_v4_stub.SERVICE_NAME, v4_stub)
    if datastore_pbs._CLOUD_DATASTORE_ENABLED:
      helper = datastore_pbs.googledatastore.helper
      credential_env = helper._DATASTORE_USE_STUB_CREDENTIAL_FOR_TEST_ENV
      os.environ[credential_env] = 'True'
      cloud_stub = cloud_datastore_v1_stub.CloudDatastoreV1Stub(
          full_app_id.get())
      self._register_stub(cloud_datastore_v1_stub.SERVICE_NAME, cloud_stub)

  def _deactivate_datastore_v3_stub(self, stub):
    stub.Write()

  def init_files_stub(self, enable=True):
    """Enables the Files API stub.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.

    Raises:
      StubNotSupportedError: If called.
    """
    raise StubNotSupportedError('The files stub is not supported in Titanoboa.')

  def init_images_stub(self, enable=True, **stub_kwargs):
    """Enables the images stub.

    The images service stub is only available in `dev_appserver` because it uses
    the PIL library.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.
      **stub_kwargs: Keyword arguments passed on to the service stub.
    """
    if not enable:
      self._disable_stub(IMAGES_SERVICE_NAME)
      return
    stub = images_stub.ImagesServiceStub(**stub_kwargs)
    self._register_stub(IMAGES_SERVICE_NAME, stub)

  def init_mail_stub(self, enable=True, **stub_kw_args):
    """Enables the mail stub.

    The email service stub is only available in `dev_appserver` because it uses
    the `subprocess` module.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.
      **stub_kw_args: Keyword arguments that are passed on to the service
          stub.
    """
    if not enable:
      self._disable_stub(MAIL_SERVICE_NAME)
      return
    stub = mail_stub.MailServiceStub(**stub_kw_args)
    self._register_stub(MAIL_SERVICE_NAME, stub)

  def init_memcache_stub(self, enable=True):
    """Enables the memcache stub.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.
    """
    if not enable:
      self._disable_stub(MEMCACHE_SERVICE_NAME)
      return
    stub = memcache_stub.MemcacheServiceStub()
    self._register_stub(MEMCACHE_SERVICE_NAME, stub)

  def init_taskqueue_stub(self, enable=True, **stub_kw_args):
    """Enables the taskqueue stub.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.
      **stub_kw_args: Keyword arguments passed on to the service stub.
    """
    if self._use_datastore_emulator:
      self._disable_stub(TASKQUEUE_SERVICE_NAME)
      delegate_stub = (
          remote_api_stub.TaskqueueStubTestbedDelegate(self.rpc_server, '/'))
      delegate_stub.SetUpStub(**stub_kw_args)
      self._test_stub_map.RegisterStub(
          TASKQUEUE_SERVICE_NAME, delegate_stub)
      self._enabled_stubs[TASKQUEUE_SERVICE_NAME] = ActiveStub(delegate_stub)
      return
    if not enable:
      self._disable_stub(TASKQUEUE_SERVICE_NAME)
      return
    stub = taskqueue_stub.TaskQueueServiceStub(**stub_kw_args)
    self._register_stub(TASKQUEUE_SERVICE_NAME, stub)

  def init_urlfetch_stub(self, enable=True, urlmatchers=None):
    """Enables the `urlfetch` stub.

    The `urlfetch` service stub uses the `urllib` module to make requests. On
    appserver, `urllib` also relies the `urlfetch` infrastructure, so using this
    stub will have no effect.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.
      urlmatchers: optional initial sequence of `(matcher, fetcher)` pairs to
          populate `urlmatchers_to_fetch_functions`; matchers passed here, if
          any, take precedence over default matchers dispatching GCS access.
    """
    if not enable:
      self._disable_stub(URLFETCH_SERVICE_NAME)
      return
    urlmatchers_to_fetch_functions = []
    if urlmatchers:
      urlmatchers_to_fetch_functions.extend(urlmatchers)
    urlmatchers_to_fetch_functions.extend(URLMATCHERS_TO_FETCH_FUNCTIONS)
    stub = urlfetch_stub.URLFetchServiceStub(
        urlmatchers_to_fetch_functions=urlmatchers_to_fetch_functions)
    self._register_stub(URLFETCH_SERVICE_NAME, stub)

  def init_user_stub(self, enable=True, **stub_kw_args):
    """Enables the users stub.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.
      **stub_kw_args: Keyword arguments that are passed on to the service
          stub.
    """
    if not enable:
      self._disable_stub(USER_SERVICE_NAME)
      return
    stub = user_service_stub.UserServiceStub(**stub_kw_args)
    self._register_stub(USER_SERVICE_NAME, stub)

  def init_xmpp_stub(self, enable=True):
    """Enables the xmpp stub.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.

    Raises:
      StubNotSupportedError: If called.
    """
    raise StubNotSupportedError('The xmpp stub is not supported in Titanoboa.')

  def init_modules_stub(self, enable=True):
    """Enables the modules stub.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.
    """
    if not enable:
      self._disable_stub(MODULES_SERVICE_NAME)
      return

    stub = modules_stub.ModulesServiceStub(request_info._LocalRequestInfo())
    self._register_stub(MODULES_SERVICE_NAME, stub)

  def _init_stub(self, service_name, *args, **kwargs):
    """Enables a stub by service name.

    Args:
      service_name: Name of the service to initialize.  This name should be
          the name used by the service stub.
      *args: Arguments to be passed to the service stub.
      **kwargs: Keyword arguments to be passed to the service stub.

      Additional arguments are passed along to the specific stub initializer.

    Raises:
      NotActivatedError: When this function is called before testbed is
          activated or after it is deactivated.
      StubNotSupportedError: When an unsupported `service_name` is provided.
    """
    if not self._activated:
      raise NotActivatedError('The testbed is not activated.')
    method_name = INIT_STUB_METHOD_NAMES.get(service_name, None)
    if method_name is None:
      msg = 'The "%s" service is not supported by testbed' % service_name
      raise StubNotSupportedError(msg)

    method = getattr(self, method_name)
    method(*args, **kwargs)

  def init_all_stubs(self, enable=True):
    """Enables all known testbed stubs.

    Args:
      enable: `True` if the fake service should be enabled, or `False` if the
          real service should be disabled.
    """
    for service_name in SUPPORTED_SERVICES:
      self._init_stub(service_name, enable)

_PROJECT_ID_VALID_CHARS = re.compile(r'[a-z0-9\-]+')


def validate_project_id(project_id):
  """Ensure that a GCP project ID is valid.

  From
  https://cloud.google.com/resource-manager/docs/creating-managing-projects#before_you_begin
  The project ID must be a unique string of 6 to 30 lowercase letters, digits,
  or hyphens. It must start with a letter, and cannot have a trailing hyphen.

  Args:
    project_id (str): project id, potentially including domain prefix, to
      validate.
  """

  def err(msg):
    raise ValueError(f'Invalid GCP project ID "{project_id}": {msg}.')

  if '~' in project_id:
    err('partition prefex (e.g. "s~") not allowed in project ID '
        '(consider gae_application instead)')

  chunks = project_id.split(':')
  if len(chunks) > 2:
    err('at most one colon is allowed (for domain prefix)')


  project_id = chunks[-1]

  if len(project_id) < 6:
    err('must be at least 6 characters')

  if len(project_id) > 30:
    err('must not be longer than 30 characters')

  if not _PROJECT_ID_VALID_CHARS.fullmatch(project_id):
    err('only lowercase letters, digits, or hyphens are allowed')

  if not project_id[0].islower():
    err('must start with a lowercase letter')

  if project_id[-1] == '-':
    err('must not end with a hyphen')



#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#





"""Key range representation and splitting."""

import json

import six
from six.moves import range

from google.appengine.api import datastore
from google.appengine.api import namespace_manager
from google.appengine.ext import db
from google.appengine.ext import ndb
from google.appengine.api import cmp_compat





class Error(Exception):
  """Base class for exceptions in this module."""


class KeyRangeError(Error):
  """Error while trying to generate a KeyRange."""


def _IsNdbQuery(query):
  return ndb is not None and isinstance(query, ndb.Query)


@cmp_compat.total_ordering_from_cmp
class KeyRange(object):
  """Represents a range of keys in the datastore.

  A KeyRange object represents a key range
    (key_start, include_start, key_end, include_end)
  and a scan direction (KeyRange.DESC or KeyRange.ASC).
  """


  DESC = "DESC"
  ASC = "ASC"

  def __init__(self,
               key_start=None,
               key_end=None,
               direction=None,
               include_start=True,
               include_end=True,
               namespace=None,
               _app=None):
    """Initialize a KeyRange object.

    Args:
      key_start: The starting key for this range (db.Key or ndb.Key).
      key_end: The ending key for this range (db.Key or ndb.Key).
      direction: The direction of the query for this range.
      include_start: Whether the start key should be included in the range.
      include_end: Whether the end key should be included in the range.
      namespace: The namespace for this range. If None then the current
          namespace is used.

    NOTE: If NDB keys are passed in, they are converted to db.Key
    instances before being stored.
    """




    if direction is None:
      direction = KeyRange.ASC
    assert direction in (KeyRange.ASC, KeyRange.DESC)
    self.direction = direction


    if ndb is not None:
      if isinstance(key_start, ndb.Key):
        key_start = key_start.to_old_key()
      if isinstance(key_end, ndb.Key):
        key_end = key_end.to_old_key()
    self.key_start = key_start
    self.key_end = key_end
    self.include_start = include_start
    self.include_end = include_end
    if namespace is not None:
      self.namespace = namespace
    else:
      self.namespace = namespace_manager.get_namespace()
    self._app = _app

  def __str__(self):
    if self.include_start:
      left_side = "["
    else:
      left_side = "("
    if self.include_end:
      right_side = "]"
    else:
      right_side = ")"
    return "%s%s%r to %r%s" % (self.direction, left_side, self.key_start,
                               self.key_end, right_side)

  def __repr__(self):
    return ("key_range.KeyRange(key_start=%r,key_end=%r,direction=%r,"
            "include_start=%r,include_end=%r, namespace=%r)") % (
                self.key_start,
                self.key_end,
                self.direction,
                self.include_start,
                self.include_end,
                self.namespace)

  def advance(self, key):
    """Updates the start of the range immediately past the specified key.

    Args:
      key: A db.Key or ndb.Key.
    """
    self.include_start = False
    if ndb is not None:
      if isinstance(key, ndb.Key):
        key = key.to_old_key()
    self.key_start = key

  def filter_query(self, query, filters=None):
    """Add query filter to restrict to this key range.

    Args:
      query: A db.Query or ndb.Query instance.
      filters: optional list of filters to apply to the query. Each filter is
        a tuple: (<property_name_as_str>, <query_operation_as_str>, <value>).
        User filters are applied first.

    Returns:
      The input query restricted to this key range.
    """
    if ndb is not None:
      if _IsNdbQuery(query):
        return self.filter_ndb_query(query, filters=filters)
    assert not _IsNdbQuery(query)


    if filters:
      for f in filters:
        query.filter("%s %s" % (f[0], f[1]), f[2])

    if self.include_start:
      start_comparator = ">="
    else:
      start_comparator = ">"
    if self.include_end:
      end_comparator = "<="
    else:
      end_comparator = "<"
    if self.key_start:
      query.filter("__key__ %s" % start_comparator, self.key_start)
    if self.key_end:
      query.filter("__key__ %s" % end_comparator, self.key_end)
    return query

  def filter_ndb_query(self, query, filters=None):
    """Add query filter to restrict to this key range.

    Args:
      query: An ndb.Query instance.
      filters: optional list of filters to apply to the query. Each filter is
        a tuple: (<property_name_as_str>, <query_operation_as_str>, <value>).
        User filters are applied first.

    Returns:
      The input query restricted to this key range.
    """
    assert _IsNdbQuery(query)


    if filters:
      for f in filters:
        query = query.filter(ndb.FilterNode(*f))

    if self.include_start:
      start_comparator = ">="
    else:
      start_comparator = ">"
    if self.include_end:
      end_comparator = "<="
    else:
      end_comparator = "<"
    if self.key_start:
      query = query.filter(ndb.FilterNode("__key__",
                                          start_comparator,
                                          self.key_start))
    if self.key_end:
      query = query.filter(ndb.FilterNode("__key__",
                                          end_comparator,
                                          self.key_end))
    return query

  def filter_datastore_query(self, query, filters=None):
    """Add query filter to restrict to this key range.

    Args:
      query: A datastore.Query instance.
      filters: optional list of filters to apply to the query. Each filter is
        a tuple: (<property_name_as_str>, <query_operation_as_str>, <value>).
        User filters are applied first.

    Returns:
      The input query restricted to this key range.
    """
    assert isinstance(query, datastore.Query)

    if filters:
      for f in filters:
        query.update({"%s %s" % (f[0], f[1]): f[2]})

    if self.include_start:
      start_comparator = ">="
    else:
      start_comparator = ">"
    if self.include_end:
      end_comparator = "<="
    else:
      end_comparator = "<"
    if self.key_start:
      query.update({"__key__ %s" % start_comparator: self.key_start})
    if self.key_end:
      query.update({"__key__ %s" % end_comparator: self.key_end})
    return query

  def __get_direction(self, asc, desc):
    """Check that self.direction is in (KeyRange.ASC, KeyRange.DESC).

    Args:
      asc: Argument to return if self.direction is KeyRange.ASC
      desc: Argument to return if self.direction is KeyRange.DESC

    Returns:
      asc or desc appropriately

    Raises:
      KeyRangeError: if self.direction is not in (KeyRange.ASC, KeyRange.DESC).
    """
    if self.direction == KeyRange.ASC:
      return asc
    elif self.direction == KeyRange.DESC:
      return desc
    else:
      raise KeyRangeError("KeyRange direction unexpected: %s", self.direction)

  def make_directed_query(self, kind_class, keys_only=False):
    """Construct a query for this key range, including the scan direction.

    Args:
      kind_class: A kind implementation class (a subclass of either
        db.Model or ndb.Model).
      keys_only: bool, default False, use keys_only on Query?

    Returns:
      A db.Query or ndb.Query instance (corresponding to kind_class).

    Raises:
      KeyRangeError: if self.direction is not in (KeyRange.ASC, KeyRange.DESC).
    """
    if ndb is not None:
      if issubclass(kind_class, ndb.Model):
        return self.make_directed_ndb_query(kind_class, keys_only=keys_only)
    assert self._app is None, '_app is not supported for db.Query'
    direction = self.__get_direction("", "-")
    query = db.Query(kind_class, namespace=self.namespace, keys_only=keys_only)
    query.order("%s__key__" % direction)

    query = self.filter_query(query)
    return query

  def make_directed_ndb_query(self, kind_class, keys_only=False):
    """Construct an NDB query for this key range, including the scan direction.

    Args:
      kind_class: An ndb.Model subclass.
      keys_only: bool, default False, use keys_only on Query?

    Returns:
      An ndb.Query instance.

    Raises:
      KeyRangeError: if self.direction is not in (KeyRange.ASC, KeyRange.DESC).
    """
    assert issubclass(kind_class, ndb.Model)
    if keys_only:
      default_options = ndb.QueryOptions(keys_only=True)
    else:
      default_options = None
    query = kind_class.query(app=self._app,
                             namespace=self.namespace,
                             default_options=default_options)
    query = self.filter_ndb_query(query)
    if self.__get_direction(True, False):
      query = query.order(kind_class._key)
    else:
      query = query.order(-kind_class._key)
    return query

  def make_directed_datastore_query(self, kind, keys_only=False):
    """Construct a query for this key range, including the scan direction.

    Args:
      kind: A string.
      keys_only: bool, default False, use keys_only on Query?

    Returns:
      A datastore.Query instance.

    Raises:
      KeyRangeError: if self.direction is not in (KeyRange.ASC, KeyRange.DESC).
    """
    direction = self.__get_direction(datastore.Query.ASCENDING,
                                     datastore.Query.DESCENDING)
    query = datastore.Query(kind, _app=self._app, keys_only=keys_only)
    query.Order(("__key__", direction))

    query = self.filter_datastore_query(query)
    return query

  def make_ascending_query(self, kind_class, keys_only=False, filters=None):
    """Construct a query for this key range without setting the scan direction.

    Args:
      kind_class: A kind implementation class (a subclass of either
        db.Model or ndb.Model).
      keys_only: bool, default False, query only for keys.
      filters: optional list of filters to apply to the query. Each filter is
        a tuple: (<property_name_as_str>, <query_operation_as_str>, <value>).
        User filters are applied first.

    Returns:
      A db.Query or ndb.Query instance (corresponding to kind_class).
    """
    if ndb is not None:
      if issubclass(kind_class, ndb.Model):
        return self.make_ascending_ndb_query(
            kind_class, keys_only=keys_only, filters=filters)
    assert self._app is None, '_app is not supported for db.Query'
    query = db.Query(kind_class, namespace=self.namespace, keys_only=keys_only)
    query.order("__key__")

    query = self.filter_query(query, filters=filters)
    return query

  def make_ascending_ndb_query(self, kind_class, keys_only=False, filters=None):
    """Construct an NDB query for this key range, without the scan direction.

    Args:
      kind_class: An ndb.Model subclass.
      keys_only: bool, default False, query only for keys.

    Returns:
      An ndb.Query instance.
    """
    assert issubclass(kind_class, ndb.Model)
    if keys_only:
      default_options = ndb.QueryOptions(keys_only=True)
    else:
      default_options = None
    query = kind_class.query(app=self._app,
                             namespace=self.namespace,
                             default_options=default_options)
    query = self.filter_ndb_query(query, filters=filters)
    query = query.order(kind_class._key)
    return query

  def make_ascending_datastore_query(self, kind, keys_only=False, filters=None):
    """Construct a query for this key range without setting the scan direction.

    Args:
      kind: A string.
      keys_only: bool, default False, use keys_only on Query?
      filters: optional list of filters to apply to the query. Each filter is
        a tuple: (<property_name_as_str>, <query_operation_as_str>, <value>).
        User filters are applied first.

    Returns:
      A datastore.Query instance.
    """
    query = datastore.Query(kind,
                            namespace=self.namespace,
                            _app=self._app,
                            keys_only=keys_only)
    query.Order(("__key__", datastore.Query.ASCENDING))

    query = self.filter_datastore_query(query, filters=filters)
    return query

  def split_range(self, batch_size=0):
    """Split this key range into a list of at most two ranges.

    This method attempts to split the key range approximately in half.
    Numeric ranges are split in the middle into two equal ranges and
    string ranges are split lexicographically in the middle.  If the
    key range is smaller than batch_size it is left unsplit.

    Note that splitting is done without knowledge of the distribution
    of actual entities in the key range, so there is no guarantee (nor
    any particular reason to believe) that the entities of the range
    are evenly split.

    Args:
      batch_size: The maximum size of a key range that should not be split.

    Returns:
      A list of one or two key ranges covering the same space as this range.
    """
    key_start = self.key_start
    key_end = self.key_end
    include_start = self.include_start
    include_end = self.include_end

    key_pairs = []
    if not key_start:
      key_pairs.append((key_start, include_start, key_end, include_end,
                        KeyRange.ASC))
    elif not key_end:
      key_pairs.append((key_start, include_start, key_end, include_end,
                        KeyRange.DESC))
    else:
      key_split = KeyRange.split_keys(key_start, key_end, batch_size)
      first_include_end = True

      if key_split == key_start:
        first_include_end = first_include_end and include_start

      key_pairs.append((key_start, include_start,
                        key_split, first_include_end,
                        KeyRange.DESC))

      second_include_end = include_end

      if key_split == key_end:
        second_include_end = False
      key_pairs.append((key_split, False,
                        key_end, second_include_end,
                        KeyRange.ASC))

    ranges = [KeyRange(key_start=start,
                       include_start=include_start,
                       key_end=end,
                       include_end=include_end,
                       direction=direction,
                       namespace=self.namespace,
                       _app=self._app)
              for (start, include_start, end, include_end, direction)
              in key_pairs]

    return ranges

  def __hash__(self):
    raise TypeError('KeyRange is unhashable')

  def __cmp__(self, other):
    """Compare two key ranges.

    Key ranges with a value of None for key_start or key_end, are always
    considered to have include_start=False or include_end=False, respectively,
    when comparing.  Since None indicates an unbounded side of the range,
    the include specifier is meaningless.  The ordering generated is total
    but somewhat arbitrary.

    Args:
      other: An object to compare to this one.

    Returns:
      -1: if this key range is less than other.
      0:  if this key range is equal to other.
      1: if this key range is greater than other.
    """
    if not isinstance(other, KeyRange):
      return 1

    self_list = [self.key_start, self.key_end, self.direction,
                 self.include_start, self.include_end, self._app,
                 self.namespace]
    if not self.key_start:
      self_list[3] = False
    if not self.key_end:
      self_list[4] = False

    other_list = [other.key_start,
                  other.key_end,
                  other.direction,
                  other.include_start,
                  other.include_end,
                  other._app,
                  other.namespace]
    if not other.key_start:
      other_list[3] = False
    if not other.key_end:
      other_list[4] = False

    return cmp_compat.cmp(self_list, other_list)

  @staticmethod
  def bisect_string_range(start, end):
    """Returns a string that is approximately in the middle of the range.

    (start, end) is treated as a string range, and it is assumed
    start <= end in the usual lexicographic string ordering. The output key
    mid is guaranteed to satisfy start <= mid <= end.

    The method proceeds by comparing initial characters of start and
    end.  When the characters are equal, they are appended to the mid
    string.  In the first place that the characters differ, the
    difference characters are averaged and this average is appended to
    the mid string.  If averaging resulted in rounding down, and
    additional character is added to the mid string to make up for the
    rounding down.  This extra step is necessary for correctness in
    the case that the average of the two characters is equal to the
    character in the start string.

    This method makes the assumption that most keys are ascii and it
    attempts to perform splitting within the ascii range when that
    results in a valid split.

    Args:
      start: A string.
      end: A string such that start <= end.

    Returns:
      A string mid such that start <= mid <= end.
    """
    if start == end:
      return start
    start += "\0"
    end += "\0"
    midpoint = []


    expected_max = 127
    for i in range(min(len(start), len(end))):
      if start[i] == end[i]:
        midpoint.append(start[i])
      else:
        ord_sum = ord(start[i]) + ord(end[i])
        midpoint.append(six.unichr(ord_sum // 2))
        if ord_sum % 2:
          if len(start) > i + 1:
            ord_start = ord(start[i+1])
          else:
            ord_start = 0
          if ord_start < expected_max:


            ord_split = (expected_max + ord_start) // 2
          else:

            ord_split = (0xFFFF + ord_start) // 2
          midpoint.append(six.unichr(ord_split))
        break
    return "".join(midpoint)

  @staticmethod
  def split_keys(key_start, key_end, batch_size):
    """Return a key that is between key_start and key_end inclusive.

    This method compares components of the ancestor paths of key_start
    and key_end.  The first place in the path that differs is
    approximately split in half.  If the kind components differ, a new
    non-existent kind halfway between the two is used to split the
    space. If the id_or_name components differ, then a new id_or_name
    that is halfway between the two is selected.  If the lower
    id_or_name is numeric and the upper id_or_name is a string, then
    the minumum string key u'\0' is used as the split id_or_name.  The
    key that is returned is the shared portion of the ancestor path
    followed by the generated split component.

    Args:
      key_start: A db.Key or ndb.Key instance for the lower end of a range.
      key_end: A db.Key or ndb.Key instance for the upper end of a range.
      batch_size: The maximum size of a range that should not be split.

    Returns:
      A db.Key instance, k, such that key_start <= k <= key_end.

    NOTE: Even though ndb.Key instances are accepted as arguments,
    the return value is always a db.Key instance.
    """
    if ndb is not None:


      if isinstance(key_start, ndb.Key):
        key_start = key_start.to_old_key()
      if isinstance(key_end, ndb.Key):
        key_end = key_end.to_old_key()
    assert key_start.app() == key_end.app()
    assert key_start.namespace() == key_end.namespace()
    path1 = key_start.to_path()
    path2 = key_end.to_path()
    len1 = len(path1)
    len2 = len(path2)
    assert len1 % 2 == 0
    assert len2 % 2 == 0
    out_path = []
    min_path_len = min(len1, len2) // 2
    for i in range(min_path_len):
      kind1 = path1[2*i]
      kind2 = path2[2*i]

      if kind1 != kind2:
        split_kind = KeyRange.bisect_string_range(kind1, kind2)
        out_path.append(split_kind)
        out_path.append(six.unichr(0))
        break




      last = (len1 == len2 == 2*(i + 1))

      id_or_name1 = path1[2*i + 1]
      id_or_name2 = path2[2*i + 1]
      id_or_name_split = KeyRange._split_id_or_name(
          id_or_name1, id_or_name2, batch_size, last)
      if id_or_name1 == id_or_name_split:
        out_path.append(kind1)
        out_path.append(id_or_name1)
      else:
        out_path.append(kind1)
        out_path.append(id_or_name_split)
        break

    return db.Key.from_path(
        *out_path,
        **{"_app": key_start.app(), "namespace": key_start.namespace()})

  @staticmethod
  def _split_id_or_name(id_or_name1, id_or_name2, batch_size, maintain_batches):
    """Return an id_or_name that is between id_or_name1 an id_or_name2.

    Attempts to split the range [id_or_name1, id_or_name2] in half,
    unless maintain_batches is true and the size of the range
    [id_or_name1, id_or_name2] is less than or equal to batch_size.

    Args:
      id_or_name1: A number or string or the id_or_name component of a key
      id_or_name2: A number or string or the id_or_name component of a key
      batch_size: The range size that will not be split if maintain_batches
        is true.
      maintain_batches: A boolean for whether to keep small ranges intact.

    Returns:
      An id_or_name such that id_or_name1 <= id_or_name <= id_or_name2.
    """
    if (isinstance(id_or_name1, six.integer_types) and
        isinstance(id_or_name2, six.integer_types)):
      if not maintain_batches or id_or_name2 - id_or_name1 > batch_size:
        return (id_or_name1 + id_or_name2) // 2
      else:
        return id_or_name1
    elif (isinstance(id_or_name1, six.string_types) and
          isinstance(id_or_name2, six.string_types)):
      return KeyRange.bisect_string_range(id_or_name1, id_or_name2)
    else:
      if (not isinstance(id_or_name1, six.integer_types) or
          not isinstance(id_or_name2, six.string_types)):
        raise KeyRangeError("Wrong key order: %r, %r" %
                            (id_or_name1, id_or_name2))

      zero_ch = six.unichr(0)
      if id_or_name2 == zero_ch:
        return (id_or_name1 + 2**63 - 1) // 2
      return zero_ch

  @staticmethod
  def guess_end_key(kind,
                    key_start,
                    probe_count=30,
                    split_rate=5):
    """Guess the end of a key range with a binary search of probe queries.

    When the 'key_start' parameter has a key hierarchy, this function will
    only determine the key range for keys in a similar hierarchy. That means
    if the keys are in the form:

      kind=Foo, name=bar/kind=Stuff, name=meep

    only this range will be probed:

      kind=Foo, name=*/kind=Stuff, name=*

    That means other entities of kind 'Stuff' that are children of another
    parent entity kind will be skipped:

      kind=Other, name=cookie/kind=Stuff, name=meep

    Args:
      key_start: The starting key of the search range. In most cases this
        should be id = 0 or name = '\0'.  May be db.Key or ndb.Key.
      kind: String name of the entity kind.
      probe_count: Optional, how many probe queries to run.
      split_rate: Exponential rate to use for splitting the range on the
        way down from the full key space. For smaller ranges this should
        be higher so more of the keyspace is skipped on initial descent.

    Returns:
      db.Key that is guaranteed to be as high or higher than the
      highest key existing for this Kind. Doing a query between 'key_start' and
      this returned Key (inclusive) will contain all entities of this Kind.

    NOTE: Even though an ndb.Key instance is accepted as argument,
    the return value is always a db.Key instance.
    """
    if ndb is not None:


      if isinstance(key_start, ndb.Key):
        key_start = key_start.to_old_key()
    app = key_start.app()
    namespace = key_start.namespace()

    full_path = key_start.to_path()
    for index, piece in enumerate(full_path):
      if index % 2 == 0:

        continue
      elif isinstance(piece, six.string_types):

        full_path[index] = u"\xffff"
      else:

        full_path[index] = 2**63 - 1

    key_end = db.Key.from_path(*full_path,
                               **{"_app": app, "namespace": namespace})
    split_key = key_end

    for _ in range(probe_count):
      for _ in range(split_rate):
        split_key = KeyRange.split_keys(key_start, split_key, 1)
      results = datastore.Query(
          kind,
          {"__key__ >": split_key},
          namespace=namespace,
          _app=app,
          keys_only=True).Get(1)
      if results:
        if results[0].name() and not key_start.name():


          return KeyRange.guess_end_key(
              kind, results[0], probe_count - 1, split_rate)
        else:
          split_rate = 1
          key_start = results[0]
          split_key = key_end
      else:
        key_end = split_key

    return key_end

  @classmethod
  def compute_split_points(cls, kind, count):
    """Computes a set of KeyRanges that are split points for a kind.

    Args:
      kind: String with the entity kind to split.
      count: Number of non-overlapping KeyRanges to generate.

    Returns:
      A list of KeyRange objects that are non-overlapping. At most "count" + 1
      KeyRange objects will be returned. At least one will be returned.
    """
    query = datastore.Query(kind=kind, keys_only=True)
    query.Order("__scatter__")
    random_keys = query.Get(count)

    if not random_keys:

      return [cls()]

    random_keys.sort()



    key_ranges = []


    key_ranges.append(cls(
        key_start=None,
        key_end=random_keys[0],
        direction=cls.ASC,
        include_start=False,
        include_end=False))


    for i in range(0, len(random_keys) - 1):
      key_ranges.append(cls(
          key_start=random_keys[i],
          key_end=random_keys[i + 1],
          direction=cls.ASC,
          include_start=True,
          include_end=False))


    key_ranges.append(cls(
        key_start=random_keys[-1],
        key_end=None,
        direction=cls.ASC,
        include_start=True,
        include_end=False))

    return key_ranges

  def to_json(self):
    """Serialize KeyRange to json.

    Returns:
      string with KeyRange json representation.
    """

    def key_to_str(key):
      if key:
        return str(key)
      else:
        return None

    obj_dict = {
        "direction": self.direction,
        "key_start": key_to_str(self.key_start),
        "key_end": key_to_str(self.key_end),
        "include_start": self.include_start,
        "include_end": self.include_end,
        "namespace": self.namespace,
        }
    if self._app:
      obj_dict["_app"] = self._app

    return json.dumps(obj_dict, sort_keys=True)


  @staticmethod
  def from_json(json_str):
    """Deserialize KeyRange from its json representation.

    Args:
      json_str: string with json representation created by key_range_to_json.

    Returns:
      deserialized KeyRange instance.
    """

    def key_from_str(key_str):
      if key_str:
        return db.Key(key_str)
      else:
        return None

    obj = json.loads(json_str)
    return KeyRange(key_from_str(obj["key_start"]),
                    key_from_str(obj["key_end"]),
                    obj["direction"],
                    obj["include_start"],
                    obj["include_end"],
                    obj.get("namespace"),
                    _app=obj.get("_app"))



#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Definition of BlobStorage base class."""


class BlobStorage(object):
  """Base class for defining how blobs are stored.

  This base class merely defines an interface that all stub blob-storage
  mechanisms must implement.
  """

  def StoreBlob(self, blob_key, blob_stream):
    """Store blob stream.

    Implement this method to persist blob data.

    Args:
      blob_key: Blob key of blob to store.
      blob_stream: Stream or stream-like object that will generate blob content.
    """
    raise NotImplementedError('Storage class must override StoreBlob method.')

  def OpenBlob(self, blob_key):
    """Open blob for streaming.

    Args:
      blob_key: Blob-key of existing blob to open for reading.

    Returns:
      Open file stream for reading blob. Caller is responsible for closing file.
    """
    raise NotImplementedError('Storage class must override OpenBlob method.')

  def DeleteBlob(self, blob_key):
    """Delete blob data from storage.

    Args:
      blob_key: Blob-key of existing blob to delete.
    """
    raise NotImplementedError('Storage class must override DeleteBlob method.')



#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Exceptions raised by the Mail API."""




class Error(Exception):
  """Base Mail error type."""


class BadRequestError(Error):
  """The email is not valid."""


class InvalidSenderError(Error):
  """The sender is not permitted to send mail for this application."""


class InvalidEmailError(Error):
  """The specified email is invalid."""


class InvalidAttachmentTypeError(Error):
  """The file type of the attachment is invalid."""


class InvalidHeaderNameError(Error):
  """The header name is invalid."""


class MissingRecipientsError(Error):
  """A recipient was not specified in the message."""


class MissingSenderError(Error):
  """A sender was not specified in the message."""


class MissingSubjectError(Error):
  """The subject was not specified in the message."""


class MissingBodyError(Error):
  """A body was not specified in the message."""


class PayloadEncodingError(Error):
  """The payload encoding is unknown."""


class UnknownEncodingError(PayloadEncodingError):
  """The encoding is unknown."""


class UnknownCharsetError(PayloadEncodingError):
  """The character set is unknown."""



#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database


_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'google/appengine/api/user_service.proto\x12\x10google.appengine\"\x99\x01\n\x10UserServiceError\"\x84\x01\n\tErrorCode\x12\x06\n\x02OK\x10\x00\x12\x19\n\x15REDIRECT_URL_TOO_LONG\x10\x01\x12\x0f\n\x0bNOT_ALLOWED\x10\x02\x12\x17\n\x13OAUTH_INVALID_TOKEN\x10\x03\x12\x19\n\x15OAUTH_INVALID_REQUEST\x10\x04\x12\x0f\n\x0bOAUTH_ERROR\x10\x05\"a\n\x15\x43reateLoginURLRequest\x12\x17\n\x0f\x64\x65stination_url\x18\x01 \x02(\t\x12\x13\n\x0b\x61uth_domain\x18\x02 \x01(\t\x12\x1a\n\x12\x66\x65\x64\x65rated_identity\x18\x03 \x01(\t\"+\n\x16\x43reateLoginURLResponse\x12\x11\n\tlogin_url\x18\x01 \x01(\t\"F\n\x16\x43reateLogoutURLRequest\x12\x17\n\x0f\x64\x65stination_url\x18\x01 \x02(\t\x12\x13\n\x0b\x61uth_domain\x18\x02 \x01(\t\"-\n\x17\x43reateLogoutURLResponse\x12\x12\n\nlogout_url\x18\x01 \x01(\t\"[\n\x13GetOAuthUserRequest\x12\r\n\x05scope\x18\x01 \x01(\t\x12\x0e\n\x06scopes\x18\x02 \x03(\t\x12%\n\x19request_writer_permission\x18\x03 \x01(\x08\x42\x02\x18\x01\"\xba\x01\n\x14GetOAuthUserResponse\x12\r\n\x05\x65mail\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x02 \x01(\t\x12\x13\n\x0b\x61uth_domain\x18\x03 \x01(\t\x12\x19\n\x11user_organization\x18\x04 \x01(\t\x12\x10\n\x08is_admin\x18\x05 \x01(\x08\x12\x11\n\tclient_id\x18\x06 \x01(\t\x12\x0e\n\x06scopes\x18\x07 \x03(\t\x12\x1d\n\x11is_project_writer\x18\x08 \x01(\x08\x42\x02\x18\x01\x42\x30\n\x1f\x63om.google.google.appengine.apiB\rUserServicePb')



_USERSERVICEERROR = DESCRIPTOR.message_types_by_name['UserServiceError']
_CREATELOGINURLREQUEST = DESCRIPTOR.message_types_by_name['CreateLoginURLRequest']
_CREATELOGINURLRESPONSE = DESCRIPTOR.message_types_by_name['CreateLoginURLResponse']
_CREATELOGOUTURLREQUEST = DESCRIPTOR.message_types_by_name['CreateLogoutURLRequest']
_CREATELOGOUTURLRESPONSE = DESCRIPTOR.message_types_by_name['CreateLogoutURLResponse']
_GETOAUTHUSERREQUEST = DESCRIPTOR.message_types_by_name['GetOAuthUserRequest']
_GETOAUTHUSERRESPONSE = DESCRIPTOR.message_types_by_name['GetOAuthUserResponse']
_USERSERVICEERROR_ERRORCODE = _USERSERVICEERROR.enum_types_by_name['ErrorCode']
UserServiceError = _reflection.GeneratedProtocolMessageType('UserServiceError', (_message.Message,), {
  'DESCRIPTOR' : _USERSERVICEERROR,
  '__module__' : 'google.appengine.api.user_service_pb2'

  })
_sym_db.RegisterMessage(UserServiceError)

CreateLoginURLRequest = _reflection.GeneratedProtocolMessageType('CreateLoginURLRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATELOGINURLREQUEST,
  '__module__' : 'google.appengine.api.user_service_pb2'

  })
_sym_db.RegisterMessage(CreateLoginURLRequest)

CreateLoginURLResponse = _reflection.GeneratedProtocolMessageType('CreateLoginURLResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATELOGINURLRESPONSE,
  '__module__' : 'google.appengine.api.user_service_pb2'

  })
_sym_db.RegisterMessage(CreateLoginURLResponse)

CreateLogoutURLRequest = _reflection.GeneratedProtocolMessageType('CreateLogoutURLRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATELOGOUTURLREQUEST,
  '__module__' : 'google.appengine.api.user_service_pb2'

  })
_sym_db.RegisterMessage(CreateLogoutURLRequest)

CreateLogoutURLResponse = _reflection.GeneratedProtocolMessageType('CreateLogoutURLResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATELOGOUTURLRESPONSE,
  '__module__' : 'google.appengine.api.user_service_pb2'

  })
_sym_db.RegisterMessage(CreateLogoutURLResponse)

GetOAuthUserRequest = _reflection.GeneratedProtocolMessageType('GetOAuthUserRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETOAUTHUSERREQUEST,
  '__module__' : 'google.appengine.api.user_service_pb2'

  })
_sym_db.RegisterMessage(GetOAuthUserRequest)

GetOAuthUserResponse = _reflection.GeneratedProtocolMessageType('GetOAuthUserResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETOAUTHUSERRESPONSE,
  '__module__' : 'google.appengine.api.user_service_pb2'

  })
_sym_db.RegisterMessage(GetOAuthUserResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\037com.google.google.appengine.apiB\rUserServicePb'
  _GETOAUTHUSERREQUEST.fields_by_name['request_writer_permission']._options = None
  _GETOAUTHUSERREQUEST.fields_by_name['request_writer_permission']._serialized_options = b'\030\001'
  _GETOAUTHUSERRESPONSE.fields_by_name['is_project_writer']._options = None
  _GETOAUTHUSERRESPONSE.fields_by_name['is_project_writer']._serialized_options = b'\030\001'
  _USERSERVICEERROR._serialized_start=62
  _USERSERVICEERROR._serialized_end=215
  _USERSERVICEERROR_ERRORCODE._serialized_start=83
  _USERSERVICEERROR_ERRORCODE._serialized_end=215
  _CREATELOGINURLREQUEST._serialized_start=217
  _CREATELOGINURLREQUEST._serialized_end=314
  _CREATELOGINURLRESPONSE._serialized_start=316
  _CREATELOGINURLRESPONSE._serialized_end=359
  _CREATELOGOUTURLREQUEST._serialized_start=361
  _CREATELOGOUTURLREQUEST._serialized_end=431
  _CREATELOGOUTURLRESPONSE._serialized_start=433
  _CREATELOGOUTURLRESPONSE._serialized_end=478
  _GETOAUTHUSERREQUEST._serialized_start=480
  _GETOAUTHUSERREQUEST._serialized_end=571
  _GETOAUTHUSERRESPONSE._serialized_start=574
  _GETOAUTHUSERRESPONSE._serialized_end=760

